@@ -5,9 +5,11 @@ namespace Illuminate\Tests\Container;
 use Illuminate\Container\Container;
 use Illuminate\Container\EntryNotFoundException;
 use Illuminate\Contracts\Container\BindingResolutionException;
+use Illuminate\Contracts\Container\CircularDependencyException;
 use PHPUnit\Framework\TestCase;
 use Psr\Container\ContainerExceptionInterface;
 use stdClass;
+use TypeError;
 
 class ContainerTest extends TestCase
 {
@@ -104,6 +106,31 @@ class ContainerTest extends TestCase
         $this->assertSame($firstInstantiation, $secondInstantiation);
     }
 
+    public function testScopedClosureResolution()
+    {
+        $container = new Container;
+        $container->scoped('class', function () {
+            return new stdClass;
+        });
+        $firstInstantiation = $container->make('class');
+        $secondInstantiation = $container->make('class');
+        $this->assertSame($firstInstantiation, $secondInstantiation);
+    }
+
+    public function testScopedClosureResets()
+    {
+        $container = new Container;
+        $container->scoped('class', function () {
+            return new stdClass;
+        });
+        $firstInstantiation = $container->make('class');
+
+        $container->forgetScopedInstances();
+
+        $secondInstantiation = $container->make('class');
+        $this->assertNotSame($firstInstantiation, $secondInstantiation);
+    }
+
     public function testAutoConcreteResolution()
     {
         $container = new Container;
@@ -120,6 +147,29 @@ class ContainerTest extends TestCase
         $this->assertSame($var1, $var2);
     }
 
+    public function testScopedConcreteResolutionResets()
+    {
+        $container = new Container;
+        $container->scoped(ContainerConcreteStub::class);
+
+        $var1 = $container->make(ContainerConcreteStub::class);
+
+        $container->forgetScopedInstances();
+
+        $var2 = $container->make(ContainerConcreteStub::class);
+
+        $this->assertNotSame($var1, $var2);
+    }
+
+    public function testBindFailsLoudlyWithInvalidArgument()
+    {
+        $this->expectException(TypeError::class);
+        $container = new Container;
+
+        $concrete = new ContainerConcreteStub;
+        $container->bind(ContainerConcreteStub::class, $concrete);
+    }
+
     public function testAbstractToConcreteResolution()
     {
         $container = new Container;
@@ -198,6 +248,15 @@ class ContainerTest extends TestCase
         $this->assertSame($bound, $resolved);
     }
 
+    public function testBindingAnInstanceAsShared()
+    {
+        $container = new Container;
+        $bound = new stdClass;
+        $container->instance('foo', $bound);
+        $object = $container->make('foo');
+        $this->assertSame($bound, $object);
+    }
+
     public function testResolutionOfDefaultParameters()
     {
         $container = new Container;
@@ -380,7 +439,7 @@ class ContainerTest extends TestCase
     {
         $container = new Container;
         $container->alias('ConcreteStub', 'foo');
-        $this->assertEquals($container->getAlias('foo'), 'ConcreteStub');
+        $this->assertSame('ConcreteStub', $container->getAlias('foo'));
     }
 
     public function testItThrowsExceptionWhenAbstractIsSameAsAlias()
@@ -406,7 +465,7 @@ class ContainerTest extends TestCase
     public function testMakeWithMethodIsAnAliasForMakeMethod()
     {
         $mock = $this->getMockBuilder(Container::class)
-                     ->setMethods(['make'])
+                     ->onlyMethods(['make'])
                      ->getMock();
 
         $mock->expects($this->once())
@@ -544,6 +603,38 @@ class ContainerTest extends TestCase
 
         $this->assertInstanceOf(ContainerConcreteStub::class, $class);
     }
+
+    // public function testContainerCanCatchCircularDependency()
+    // {
+    //     $this->expectException(CircularDependencyException::class);
+
+    //     $container = new Container;
+    //     $container->get(CircularAStub::class);
+    // }
+}
+
+class CircularAStub
+{
+    public function __construct(CircularBStub $b)
+    {
+        //
+    }
+}
+
+class CircularBStub
+{
+    public function __construct(CircularCStub $c)
+    {
+        //
+    }
+}
+
+class CircularCStub
+{
+    public function __construct(CircularAStub $a)
+    {
+        //
+    }
 }
 
 class ContainerConcreteStub
