@@ -2,17 +2,64 @@
 
 namespace Illuminate\Tests\Database;
 
+use Illuminate\Database\Capsule\Manager as DB;
+use Illuminate\Database\Eloquent\Builder;
 use Illuminate\Database\Eloquent\Collection;
 use Illuminate\Database\Eloquent\Model;
+use Illuminate\Database\Eloquent\Model as Eloquent;
 use Illuminate\Support\Collection as BaseCollection;
 use LogicException;
 use Mockery as m;
 use PHPUnit\Framework\TestCase;
+use stdClass;
 
 class DatabaseEloquentCollectionTest extends TestCase
 {
+    /**
+     * Setup the database schema.
+     *
+     * @return void
+     */
+    protected function setUp(): void
+    {
+        $db = new DB;
+
+        $db->addConnection([
+            'driver' => 'sqlite',
+            'database' => ':memory:',
+        ]);
+
+        $db->bootEloquent();
+        $db->setAsGlobal();
+
+        $this->createSchema();
+    }
+
+    protected function createSchema()
+    {
+        $this->schema()->create('users', function ($table) {
+            $table->increments('id');
+            $table->string('email')->unique();
+        });
+
+        $this->schema()->create('articles', function ($table) {
+            $table->increments('id');
+            $table->integer('user_id');
+            $table->string('title');
+        });
+
+        $this->schema()->create('comments', function ($table) {
+            $table->increments('id');
+            $table->integer('article_id');
+            $table->string('content');
+        });
+    }
+
     protected function tearDown(): void
     {
+        $this->schema()->drop('users');
+        $this->schema()->drop('articles');
+        $this->schema()->drop('comments');
         m::close();
     }
 
@@ -157,7 +204,7 @@ class DatabaseEloquentCollectionTest extends TestCase
 
     public function testLoadMethodEagerLoadsGivenRelationships()
     {
-        $c = $this->getMockBuilder(Collection::class)->setMethods(['first'])->setConstructorArgs([['foo']])->getMock();
+        $c = $this->getMockBuilder(Collection::class)->onlyMethods(['first'])->setConstructorArgs([['foo']])->getMock();
         $mockItem = m::mock(stdClass::class);
         $c->expects($this->once())->method('first')->willReturn($mockItem);
         $mockItem->shouldReceive('newQueryWithoutRelationships')->once()->andReturn($mockItem);
@@ -276,10 +323,10 @@ class DatabaseEloquentCollectionTest extends TestCase
 
     public function testCollectionReturnsDuplicateBasedOnlyOnKeys()
     {
-        $one = new TestEloquentCollectionModel();
-        $two = new TestEloquentCollectionModel();
-        $three = new TestEloquentCollectionModel();
-        $four = new TestEloquentCollectionModel();
+        $one = new TestEloquentCollectionModel;
+        $two = new TestEloquentCollectionModel;
+        $three = new TestEloquentCollectionModel;
+        $four = new TestEloquentCollectionModel;
         $one->id = 1;
         $one->someAttribute = '1';
         $two->id = 1;
@@ -344,10 +391,10 @@ class DatabaseEloquentCollectionTest extends TestCase
 
     public function testCollectionReturnsUniqueStrictBasedOnKeysOnly()
     {
-        $one = new TestEloquentCollectionModel();
-        $two = new TestEloquentCollectionModel();
-        $three = new TestEloquentCollectionModel();
-        $four = new TestEloquentCollectionModel();
+        $one = new TestEloquentCollectionModel;
+        $two = new TestEloquentCollectionModel;
+        $three = new TestEloquentCollectionModel;
+        $four = new TestEloquentCollectionModel;
         $one->id = 1;
         $one->someAttribute = '1';
         $two->id = 1;
@@ -415,6 +462,15 @@ class DatabaseEloquentCollectionTest extends TestCase
         $this->assertEquals([], $c[0]->getHidden());
     }
 
+    public function testAppendsAddsTestOnEntireCollection()
+    {
+        $c = new Collection([new TestEloquentCollectionModel]);
+        $c = $c->makeVisible('test');
+        $c = $c->append('test');
+
+        $this->assertEquals(['test' => 'test'], $c[0]->toArray());
+    }
+
     public function testNonModelRelatedMethods()
     {
         $a = new Collection([['foo' => 'bar'], ['foo' => 'baz']]);
@@ -454,30 +510,165 @@ class DatabaseEloquentCollectionTest extends TestCase
     public function testQueueableRelationshipsReturnsOnlyRelationsCommonToAllModels()
     {
         // This is needed to prevent loading non-existing relationships on polymorphic model collections (#26126)
-        $c = new Collection([new class {
-            public function getQueueableRelations()
+        $c = new Collection([
+            new class
             {
-                return ['user'];
-            }
-        }, new class {
-            public function getQueueableRelations()
+                public function getQueueableRelations()
+                {
+                    return ['user'];
+                }
+            },
+            new class
             {
-                return ['user', 'comments'];
-            }
-        }]);
+                public function getQueueableRelations()
+                {
+                    return ['user', 'comments'];
+                }
+            },
+        ]);
 
         $this->assertEquals(['user'], $c->getQueueableRelations());
     }
 
+    public function testQueueableRelationshipsIgnoreCollectionKeys()
+    {
+        $c = new Collection([
+            'foo' => new class
+            {
+                public function getQueueableRelations()
+                {
+                    return [];
+                }
+            },
+            'bar' => new class
+            {
+                public function getQueueableRelations()
+                {
+                    return [];
+                }
+            },
+        ]);
+
+        $this->assertEquals([], $c->getQueueableRelations());
+    }
+
     public function testEmptyCollectionStayEmptyOnFresh()
     {
         $c = new Collection;
         $this->assertEquals($c, $c->fresh());
     }
+
+    public function testCanConvertCollectionOfModelsToEloquentQueryBuilder()
+    {
+        $one = m::mock(Model::class);
+        $one->shouldReceive('getKey')->andReturn(1);
+
+        $two = m::mock(Model::class);
+        $two->shouldReceive('getKey')->andReturn(2);
+
+        $c = new Collection([$one, $two]);
+
+        $mocBuilder = m::mock(Builder::class);
+        $one->shouldReceive('newModelQuery')->once()->andReturn($mocBuilder);
+        $mocBuilder->shouldReceive('whereKey')->once()->with($c->modelKeys())->andReturn($mocBuilder);
+        $this->assertInstanceOf(Builder::class, $c->toQuery());
+    }
+
+    public function testConvertingEmptyCollectionToQueryThrowsException()
+    {
+        $this->expectException(LogicException::class);
+
+        $c = new Collection;
+        $c->toQuery();
+    }
+
+    public function testLoadExistsShouldCastBool()
+    {
+        $this->seedData();
+        $user = EloquentTestUserModel::with('articles')->first();
+        $user->articles->loadExists('comments');
+        $commentsExists = $user->articles->pluck('comments_exists')->toArray();
+        $this->assertContainsOnly('bool', $commentsExists);
+    }
+
+    /**
+     * Helpers...
+     */
+    protected function seedData()
+    {
+        $user = EloquentTestUserModel::create(['id' => 1, 'email' => 'taylorotwell@gmail.com']);
+
+        EloquentTestArticleModel::query()->insert([
+            ['user_id' => 1, 'title' => 'Another title'],
+            ['user_id' => 1, 'title' => 'Another title'],
+            ['user_id' => 1, 'title' => 'Another title'],
+        ]);
+
+        EloquentTestCommentModel::query()->insert([
+            ['article_id' => 1, 'content' => 'Another comment'],
+            ['article_id' => 2, 'content' => 'Another comment'],
+        ]);
+    }
+
+    /**
+     * Get a database connection instance.
+     *
+     * @return \Illuminate\Database\ConnectionInterface
+     */
+    protected function connection()
+    {
+        return Eloquent::getConnectionResolver()->connection();
+    }
+
+    /**
+     * Get a schema builder instance.
+     *
+     * @return \Illuminate\Database\Schema\Builder
+     */
+    protected function schema()
+    {
+        return $this->connection()->getSchemaBuilder();
+    }
 }
 
 class TestEloquentCollectionModel extends Model
 {
     protected $visible = ['visible'];
     protected $hidden = ['hidden'];
+
+    public function getTestAttribute()
+    {
+        return 'test';
+    }
+}
+
+class EloquentTestUserModel extends Model
+{
+    protected $table = 'users';
+    protected $guarded = [];
+    public $timestamps = false;
+
+    public function articles()
+    {
+        return $this->hasMany(EloquentTestArticleModel::class, 'user_id');
+    }
+}
+
+class EloquentTestArticleModel extends Model
+{
+    protected $table = 'articles';
+    protected $guarded = [];
+    public $timestamps = false;
+
+    public function comments()
+    {
+        return $this->hasMany(EloquentTestCommentModel::class, 'article_id');
+    }
+}
+
+class EloquentTestCommentModel extends Model
+{
+    protected $table = 'comments';
+    protected $guarded = [];
+    public $timestamps = false;
 }
