@@ -8,8 +8,10 @@ use Illuminate\Database\Eloquent\Model as Eloquent;
 use Illuminate\Database\Eloquent\SoftDeletes;
 use Illuminate\Database\Eloquent\SoftDeletingScope;
 use Illuminate\Database\Query\Builder;
+use Illuminate\Pagination\CursorPaginator;
 use Illuminate\Pagination\Paginator;
 use Illuminate\Support\Carbon;
+use Mockery;
 use PHPUnit\Framework\TestCase;
 
 class DatabaseEloquentSoftDeletesIntegrationTest extends TestCase
@@ -21,8 +23,8 @@ class DatabaseEloquentSoftDeletesIntegrationTest extends TestCase
         $db = new DB;
 
         $db->addConnection([
-            'driver'    => 'sqlite',
-            'database'  => ':memory:',
+            'driver' => 'sqlite',
+            'database' => ':memory:',
         ]);
 
         $db->bootEloquent();
@@ -136,12 +138,19 @@ class DatabaseEloquentSoftDeletesIntegrationTest extends TestCase
             return 1;
         });
 
+        CursorPaginator::currentCursorResolver(function () {
+            return null;
+        });
+
         $query = SoftDeletesTestUser::query();
         $this->assertCount(1, $query->paginate(2)->all());
 
         $query = SoftDeletesTestUser::query();
         $this->assertCount(1, $query->simplePaginate(2)->all());
 
+        $query = SoftDeletesTestUser::query();
+        $this->assertCount(1, $query->cursorPaginate(2)->all());
+
         $this->assertEquals(0, SoftDeletesTestUser::where('email', 'taylorotwell@gmail.com')->increment('id'));
         $this->assertEquals(0, SoftDeletesTestUser::where('email', 'taylorotwell@gmail.com')->decrement('id'));
     }
@@ -181,6 +190,42 @@ class DatabaseEloquentSoftDeletesIntegrationTest extends TestCase
         $this->assertEquals(1, $users->first()->id);
     }
 
+    public function testForceDeleteUpdateExistsProperty()
+    {
+        $this->createUsers();
+        $user = SoftDeletesTestUser::find(2);
+
+        $this->assertTrue($user->exists);
+
+        $user->forceDelete();
+
+        $this->assertFalse($user->exists);
+    }
+
+    public function testForceDeleteDoesntUpdateExistsPropertyIfFailed()
+    {
+        $user = new class() extends SoftDeletesTestUser
+        {
+            public $exists = true;
+
+            public function newModelQuery()
+            {
+                return Mockery::spy(parent::newModelQuery(), function (Mockery\MockInterface $mock) {
+                    $mock->shouldReceive('forceDelete')->andThrow(new \Exception());
+                });
+            }
+        };
+
+        $this->assertTrue($user->exists);
+
+        try {
+            $user->forceDelete();
+        } catch (\Exception $exception) {
+        }
+
+        $this->assertTrue($user->exists);
+    }
+
     public function testRestoreRestoresRecords()
     {
         $this->createUsers();
@@ -266,7 +311,7 @@ class DatabaseEloquentSoftDeletesIntegrationTest extends TestCase
         $now = Carbon::now();
         $this->createUsers();
 
-        /** @var SoftDeletesTestUser $userModel */
+        /** @var \Illuminate\Tests\Database\SoftDeletesTestUser $userModel */
         $userModel = SoftDeletesTestUser::find(2);
         $userModel->delete();
         $this->assertEquals($now->toDateTimeString(), $userModel->getOriginal('deleted_at'));
@@ -281,7 +326,7 @@ class DatabaseEloquentSoftDeletesIntegrationTest extends TestCase
     {
         $this->createUsers();
 
-        /** @var SoftDeletesTestUser $userModel */
+        /** @var \Illuminate\Tests\Database\SoftDeletesTestUser $userModel */
         $userModel = SoftDeletesTestUser::find(2);
         $userModel->delete();
         $userModel->restore();
@@ -296,7 +341,7 @@ class DatabaseEloquentSoftDeletesIntegrationTest extends TestCase
     {
         $this->createUsers();
 
-        /** @var SoftDeletesTestUser $userModel */
+        /** @var \Illuminate\Tests\Database\SoftDeletesTestUser $userModel */
         $userModel = SoftDeletesTestUser::withTrashed()->find(1);
         $userModel->restore();
         $this->assertEquals($userModel->deleted_at, SoftDeletesTestUser::find(1)->deleted_at);
@@ -311,7 +356,7 @@ class DatabaseEloquentSoftDeletesIntegrationTest extends TestCase
     {
         $this->createUsers();
 
-        /** @var SoftDeletesTestUser $userModel */
+        /** @var \Illuminate\Tests\Database\SoftDeletesTestUser $userModel */
         $userModel = SoftDeletesTestUser::find(2);
         $userModel->email = 'foo@bar.com';
         $userModel->delete();
