@@ -6,7 +6,8 @@ use Illuminate\Contracts\Filesystem\FileNotFoundException;
 use Illuminate\Filesystem\Filesystem;
 use Illuminate\Filesystem\FilesystemManager;
 use Illuminate\Foundation\Application;
-use Illuminate\Foundation\Testing\Assert;
+use Illuminate\Support\LazyCollection;
+use Illuminate\Testing\Assert;
 use Mockery as m;
 use PHPUnit\Framework\TestCase;
 use SplFileInfo;
@@ -20,7 +21,7 @@ class FilesystemTest extends TestCase
      */
     public static function setUpTempDir()
     {
-        self::$tempDir = __DIR__.'/tmp';
+        self::$tempDir = sys_get_temp_dir().'/tmp';
         mkdir(self::$tempDir);
     }
 
@@ -56,6 +57,27 @@ class FilesystemTest extends TestCase
         $this->assertStringEqualsFile(self::$tempDir.'/file.txt', 'Hello World');
     }
 
+    public function testLines()
+    {
+        $path = self::$tempDir.'/file.txt';
+
+        $contents = LazyCollection::times(3)
+            ->map(function ($number) {
+                return "line-{$number}";
+            })
+            ->join("\n");
+
+        file_put_contents($path, $contents);
+
+        $files = new Filesystem;
+        $this->assertInstanceOf(LazyCollection::class, $files->lines($path));
+
+        $this->assertSame(
+            ['line-1', 'line-2', 'line-3'],
+            $files->lines($path)->all()
+        );
+    }
+
     public function testReplaceCreatesFile()
     {
         $tempFile = self::$tempDir.'/file.txt';
@@ -66,12 +88,22 @@ class FilesystemTest extends TestCase
         $this->assertStringEqualsFile($tempFile, 'Hello World');
     }
 
+    public function testReplaceInFileCorrectlyReplaces()
+    {
+        $tempFile = self::$tempDir.'/file.txt';
+
+        $filesystem = new Filesystem;
+
+        $filesystem->put($tempFile, 'Hello World');
+        $filesystem->replaceInFile('Hello World', 'Hello Taylor', $tempFile);
+        $this->assertStringEqualsFile($tempFile, 'Hello Taylor');
+    }
+
+    /**
+     * @requires OS Linux|Darwin
+     */
     public function testReplaceWhenUnixSymlinkExists()
     {
-        if (windows_os()) {
-            $this->markTestSkipped('The operating system is Windows');
-        }
-
         $tempFile = self::$tempDir.'/file.txt';
         $symlinkDir = self::$tempDir.'/symlink_dir';
         $symlink = "{$symlinkDir}/symlink.txt";
@@ -115,7 +147,7 @@ class FilesystemTest extends TestCase
         $files = new Filesystem;
         $files->chmod(self::$tempDir.'/file.txt', 0755);
         $filePermission = substr(sprintf('%o', fileperms(self::$tempDir.'/file.txt')), -4);
-        $expectedPermissions = DIRECTORY_SEPARATOR == '\\' ? '0666' : '0755';
+        $expectedPermissions = DIRECTORY_SEPARATOR === '\\' ? '0666' : '0755';
         $this->assertEquals($expectedPermissions, $filePermission);
     }
 
@@ -126,7 +158,7 @@ class FilesystemTest extends TestCase
 
         $files = new Filesystem;
         $filePermission = $files->chmod(self::$tempDir.'/file.txt');
-        $expectedPermissions = DIRECTORY_SEPARATOR == '\\' ? '0666' : '0755';
+        $expectedPermissions = DIRECTORY_SEPARATOR === '\\' ? '0666' : '0755';
         $this->assertEquals($expectedPermissions, $filePermission);
     }
 
@@ -462,14 +494,10 @@ class FilesystemTest extends TestCase
 
     /**
      * @requires extension pcntl
-     * @requires function pcntl_fork
+     * @requires OS Linux|Darwin
      */
     public function testSharedGet()
     {
-        if (PHP_OS == 'Darwin') {
-            $this->markTestSkipped('The operating system is MacOS.');
-        }
-
         $content = str_repeat('123456', 1000000);
         $result = 1;
 
