@@ -1,6 +1,6 @@
 <?php
 
-namespace Illuminate\Tests\Foundation;
+namespace Illuminate\Tests\Foundation\Bootstrap;
 
 use Illuminate\Foundation\Application;
 use Illuminate\Foundation\Bootstrap\LoadEnvironmentVariables;
@@ -11,8 +11,7 @@ class LoadEnvironmentVariablesTest extends TestCase
 {
     protected function tearDown(): void
     {
-        unset($_ENV['FOO']);
-        unset($_SERVER['FOO']);
+        unset($_ENV['FOO'], $_SERVER['FOO']);
         putenv('FOO');
         m::close();
     }
