@@ -2,6 +2,7 @@
 
 namespace Illuminate\Tests\Foundation;
 
+use Illuminate\Config\Repository;
 use Illuminate\Contracts\Support\DeferrableProvider;
 use Illuminate\Foundation\Application;
 use Illuminate\Foundation\Bootstrap\RegisterFacades;
@@ -45,7 +46,8 @@ class FoundationApplicationTest extends TestCase
     public function testClassesAreBoundWhenServiceProviderIsRegistered()
     {
         $app = new Application;
-        $app->register($provider = new class($app) extends ServiceProvider {
+        $app->register($provider = new class($app) extends ServiceProvider
+        {
             public $bindings = [
                 AbstractClass::class => ConcreteClass::class,
             ];
@@ -62,7 +64,8 @@ class FoundationApplicationTest extends TestCase
     public function testSingletonsAreCreatedWhenServiceProviderIsRegistered()
     {
         $app = new Application;
-        $app->register($provider = new class($app) extends ServiceProvider {
+        $app->register($provider = new class($app) extends ServiceProvider
+        {
             public $singletons = [
                 AbstractClass::class => ConcreteClass::class,
             ];
@@ -87,6 +90,18 @@ class FoundationApplicationTest extends TestCase
         $this->assertArrayHasKey($class, $app->getLoadedProviders());
     }
 
+    public function testServiceProvidersCouldBeLoaded()
+    {
+        $provider = m::mock(ServiceProvider::class);
+        $class = get_class($provider);
+        $provider->shouldReceive('register')->once();
+        $app = new Application;
+        $app->register($provider);
+
+        $this->assertTrue($app->providerIsLoaded($class));
+        $this->assertFalse($app->providerIsLoaded(ApplicationBasicServiceProviderStub::class));
+    }
+
     public function testDeferredServicesMarkedAsBound()
     {
         $app = new Application;
@@ -133,7 +148,7 @@ class FoundationApplicationTest extends TestCase
         $app->setDeferredServices(['foo' => ApplicationDeferredServiceProviderStub::class]);
         $app->instance('foo', 'bar');
         $instance = $app->make('foo');
-        $this->assertEquals($instance, 'bar');
+        $this->assertSame('bar', $instance);
     }
 
     public function testDeferredServicesAreLazilyInitialized()
@@ -180,7 +195,7 @@ class FoundationApplicationTest extends TestCase
             SampleImplementation::class => SampleImplementationDeferredServiceProvider::class,
         ]);
         $instance = $app->make(SampleInterface::class);
-        $this->assertEquals($instance->getPrimitive(), 'foo');
+        $this->assertSame('foo', $instance->getPrimitive());
     }
 
     public function testEnvironment()
@@ -225,6 +240,19 @@ class FoundationApplicationTest extends TestCase
         $this->assertFalse($testing->isProduction());
     }
 
+    public function testDebugHelper()
+    {
+        $debugOff = new Application;
+        $debugOff['config'] = new Repository(['app' => ['debug' => false]]);
+
+        $this->assertFalse($debugOff->hasDebugModeEnabled());
+
+        $debugOn = new Application;
+        $debugOn['config'] = new Repository(['app' => ['debug' => true]]);
+
+        $this->assertTrue($debugOn->hasDebugModeEnabled());
+    }
+
     public function testMethodAfterLoadingEnvironmentAddsClosure()
     {
         $app = new Application;
@@ -363,7 +391,7 @@ class FoundationApplicationTest extends TestCase
         $this->assertSame('/base/path'.$ds.'bootstrap'.$ds.'cache/services.php', $app->getCachedServicesPath());
         $this->assertSame('/base/path'.$ds.'bootstrap'.$ds.'cache/packages.php', $app->getCachedPackagesPath());
         $this->assertSame('/base/path'.$ds.'bootstrap'.$ds.'cache/config.php', $app->getCachedConfigPath());
-        $this->assertSame('/base/path'.$ds.'bootstrap'.$ds.'cache/routes.php', $app->getCachedRoutesPath());
+        $this->assertSame('/base/path'.$ds.'bootstrap'.$ds.'cache/routes-v7.php', $app->getCachedRoutesPath());
         $this->assertSame('/base/path'.$ds.'bootstrap'.$ds.'cache/events.php', $app->getCachedEventsPath());
     }
 
@@ -376,7 +404,6 @@ class FoundationApplicationTest extends TestCase
         $_SERVER['APP_ROUTES_CACHE'] = '/absolute/path/routes.php';
         $_SERVER['APP_EVENTS_CACHE'] = '/absolute/path/events.php';
 
-        $ds = DIRECTORY_SEPARATOR;
         $this->assertSame('/absolute/path/services.php', $app->getCachedServicesPath());
         $this->assertSame('/absolute/path/packages.php', $app->getCachedPackagesPath());
         $this->assertSame('/absolute/path/config.php', $app->getCachedConfigPath());
@@ -419,7 +446,7 @@ class FoundationApplicationTest extends TestCase
 
     public function testEnvPathsAreUsedAndMadeAbsoluteForCachePathsWhenSpecifiedAsRelativeWithNullBasePath()
     {
-        $app = new Application();
+        $app = new Application;
         $_SERVER['APP_SERVICES_CACHE'] = 'relative/path/services.php';
         $_SERVER['APP_PACKAGES_CACHE'] = 'relative/path/packages.php';
         $_SERVER['APP_CONFIG_CACHE'] = 'relative/path/config.php';
@@ -441,6 +468,31 @@ class FoundationApplicationTest extends TestCase
             $_SERVER['APP_EVENTS_CACHE']
         );
     }
+
+    public function testEnvPathsAreAbsoluteInWindows()
+    {
+        $app = new Application(__DIR__);
+        $app->addAbsoluteCachePathPrefix('C:');
+        $_SERVER['APP_SERVICES_CACHE'] = 'C:\framework\services.php';
+        $_SERVER['APP_PACKAGES_CACHE'] = 'C:\framework\packages.php';
+        $_SERVER['APP_CONFIG_CACHE'] = 'C:\framework\config.php';
+        $_SERVER['APP_ROUTES_CACHE'] = 'C:\framework\routes.php';
+        $_SERVER['APP_EVENTS_CACHE'] = 'C:\framework\events.php';
+
+        $this->assertSame('C:\framework\services.php', $app->getCachedServicesPath());
+        $this->assertSame('C:\framework\packages.php', $app->getCachedPackagesPath());
+        $this->assertSame('C:\framework\config.php', $app->getCachedConfigPath());
+        $this->assertSame('C:\framework\routes.php', $app->getCachedRoutesPath());
+        $this->assertSame('C:\framework\events.php', $app->getCachedEventsPath());
+
+        unset(
+            $_SERVER['APP_SERVICES_CACHE'],
+            $_SERVER['APP_PACKAGES_CACHE'],
+            $_SERVER['APP_CONFIG_CACHE'],
+            $_SERVER['APP_ROUTES_CACHE'],
+            $_SERVER['APP_EVENTS_CACHE']
+        );
+    }
 }
 
 class ApplicationBasicServiceProviderStub extends ServiceProvider
