@@ -1,9 +1,10 @@
 <?php
 
-namespace Illuminate\Tests\Foundation\Testing\Concerns;
+namespace Illuminate\Tests\Foundation\Bootstrap\Testing\Concerns;
 
 use Illuminate\Foundation\Mix;
 use Orchestra\Testbench\TestCase;
+use stdClass;
 
 class InteractsWithContainerTest extends TestCase
 {
@@ -17,7 +18,7 @@ class InteractsWithContainerTest extends TestCase
 
     public function testWithMixRestoresOriginalHandlerAndReturnsInstance()
     {
-        $handler = new \stdClass();
+        $handler = new stdClass;
         $this->app->instance(Mix::class, $handler);
 
         $this->withoutMix();
@@ -26,4 +27,25 @@ class InteractsWithContainerTest extends TestCase
         $this->assertSame($handler, resolve(Mix::class));
         $this->assertSame($this, $instance);
     }
+
+    public function testForgetMock()
+    {
+        $this->mock(IntanceStub::class)
+            ->shouldReceive('execute')
+            ->once()
+            ->andReturn('bar');
+
+        $this->assertSame('bar', $this->app->make(IntanceStub::class)->execute());
+
+        $this->forgetMock(IntanceStub::class);
+        $this->assertSame('foo', $this->app->make(IntanceStub::class)->execute());
+    }
+}
+
+class IntanceStub
+{
+    public function execute()
+    {
+        return 'foo';
+    }
 }
