@@ -6,6 +6,9 @@ use Illuminate\Http\Request;
 use Illuminate\Http\UploadedFile;
 use Illuminate\Routing\Route;
 use Illuminate\Session\Store;
+use Illuminate\Support\Carbon;
+use Illuminate\Support\Collection;
+use InvalidArgumentException;
 use Mockery as m;
 use PHPUnit\Framework\TestCase;
 use RuntimeException;
@@ -21,7 +24,7 @@ class HttpRequestTest extends TestCase
 
     public function testInstanceMethod()
     {
-        $request = Request::create('', 'GET');
+        $request = Request::create('');
         $this->assertSame($request, $request->instance());
     }
 
@@ -57,10 +60,10 @@ class HttpRequestTest extends TestCase
 
     public function testPathMethod()
     {
-        $request = Request::create('', 'GET');
+        $request = Request::create('');
         $this->assertSame('/', $request->path());
 
-        $request = Request::create('/foo/bar', 'GET');
+        $request = Request::create('/foo/bar');
         $this->assertSame('foo/bar', $request->path());
     }
 
@@ -75,7 +78,7 @@ class HttpRequestTest extends TestCase
      */
     public function testSegmentMethod($path, $segment, $expected)
     {
-        $request = Request::create($path, 'GET');
+        $request = Request::create($path);
         $this->assertEquals($expected, $request->segment($segment, 'default'));
     }
 
@@ -94,10 +97,10 @@ class HttpRequestTest extends TestCase
      */
     public function testSegmentsMethod($path, $expected)
     {
-        $request = Request::create($path, 'GET');
+        $request = Request::create($path);
         $this->assertEquals($expected, $request->segments());
 
-        $request = Request::create('foo/bar', 'GET');
+        $request = Request::create('foo/bar');
         $this->assertEquals(['foo', 'bar'], $request->segments());
     }
 
@@ -113,60 +116,60 @@ class HttpRequestTest extends TestCase
 
     public function testUrlMethod()
     {
-        $request = Request::create('http://foo.com/foo/bar?name=taylor', 'GET');
+        $request = Request::create('http://foo.com/foo/bar?name=taylor');
         $this->assertSame('http://foo.com/foo/bar', $request->url());
 
-        $request = Request::create('http://foo.com/foo/bar/?', 'GET');
+        $request = Request::create('http://foo.com/foo/bar/?');
         $this->assertSame('http://foo.com/foo/bar', $request->url());
     }
 
     public function testFullUrlMethod()
     {
-        $request = Request::create('http://foo.com/foo/bar?name=taylor', 'GET');
+        $request = Request::create('http://foo.com/foo/bar?name=taylor');
         $this->assertSame('http://foo.com/foo/bar?name=taylor', $request->fullUrl());
 
-        $request = Request::create('https://foo.com', 'GET');
+        $request = Request::create('https://foo.com');
         $this->assertSame('https://foo.com', $request->fullUrl());
 
-        $request = Request::create('https://foo.com', 'GET');
+        $request = Request::create('https://foo.com');
         $this->assertSame('https://foo.com/?coupon=foo', $request->fullUrlWithQuery(['coupon' => 'foo']));
 
-        $request = Request::create('https://foo.com?a=b', 'GET');
+        $request = Request::create('https://foo.com?a=b');
         $this->assertSame('https://foo.com/?a=b', $request->fullUrl());
 
-        $request = Request::create('https://foo.com?a=b', 'GET');
+        $request = Request::create('https://foo.com?a=b');
         $this->assertSame('https://foo.com/?a=b&coupon=foo', $request->fullUrlWithQuery(['coupon' => 'foo']));
 
-        $request = Request::create('https://foo.com?a=b', 'GET');
+        $request = Request::create('https://foo.com?a=b');
         $this->assertSame('https://foo.com/?a=c', $request->fullUrlWithQuery(['a' => 'c']));
 
-        $request = Request::create('http://foo.com/foo/bar?name=taylor', 'GET');
+        $request = Request::create('http://foo.com/foo/bar?name=taylor');
         $this->assertSame('http://foo.com/foo/bar?name=taylor', $request->fullUrlWithQuery(['name' => 'taylor']));
 
-        $request = Request::create('http://foo.com/foo/bar/?name=taylor', 'GET');
+        $request = Request::create('http://foo.com/foo/bar/?name=taylor');
         $this->assertSame('http://foo.com/foo/bar?name=graham', $request->fullUrlWithQuery(['name' => 'graham']));
 
-        $request = Request::create('https://foo.com', 'GET');
+        $request = Request::create('https://foo.com');
         $this->assertSame('https://foo.com/?key=value%20with%20spaces', $request->fullUrlWithQuery(['key' => 'value with spaces']));
     }
 
     public function testIsMethod()
     {
-        $request = Request::create('/foo/bar', 'GET');
+        $request = Request::create('/foo/bar');
 
         $this->assertTrue($request->is('foo*'));
         $this->assertFalse($request->is('bar*'));
         $this->assertTrue($request->is('*bar*'));
         $this->assertTrue($request->is('bar*', 'foo*', 'baz'));
 
-        $request = Request::create('/', 'GET');
+        $request = Request::create('/');
 
         $this->assertTrue($request->is('/'));
     }
 
     public function testFullUrlIsMethod()
     {
-        $request = Request::create('http://example.com/foo/bar', 'GET');
+        $request = Request::create('http://example.com/foo/bar');
 
         $this->assertTrue($request->fullUrlIs('http://example.com/foo/bar'));
         $this->assertFalse($request->fullUrlIs('example.com*'));
@@ -178,7 +181,7 @@ class HttpRequestTest extends TestCase
 
     public function testRouteIsMethod()
     {
-        $request = Request::create('/foo/bar', 'GET');
+        $request = Request::create('/foo/bar');
 
         $this->assertFalse($request->routeIs('foo.bar'));
 
@@ -196,7 +199,7 @@ class HttpRequestTest extends TestCase
 
     public function testRouteMethod()
     {
-        $request = Request::create('/foo/bar', 'GET');
+        $request = Request::create('/foo/bar');
 
         $request->setRouteResolver(function () use ($request) {
             $route = new Route('GET', '/foo/{required}/{optional?}', []);
@@ -213,7 +216,7 @@ class HttpRequestTest extends TestCase
 
     public function testAjaxMethod()
     {
-        $request = Request::create('/', 'GET');
+        $request = Request::create('/');
         $this->assertFalse($request->ajax());
         $request = Request::create('/', 'GET', [], [], [], ['HTTP_X_REQUESTED_WITH' => 'XMLHttpRequest'], '{}');
         $this->assertTrue($request->ajax());
@@ -226,7 +229,7 @@ class HttpRequestTest extends TestCase
 
     public function testPrefetchMethod()
     {
-        $request = Request::create('/', 'GET');
+        $request = Request::create('/');
         $this->assertFalse($request->prefetch());
 
         $request->server->set('HTTP_X_MOZ', '');
@@ -260,9 +263,9 @@ class HttpRequestTest extends TestCase
 
     public function testSecureMethod()
     {
-        $request = Request::create('http://example.com', 'GET');
+        $request = Request::create('http://example.com');
         $this->assertFalse($request->secure());
-        $request = Request::create('https://example.com', 'GET');
+        $request = Request::create('https://example.com');
         $this->assertTrue($request->secure());
     }
 
@@ -300,6 +303,76 @@ class HttpRequestTest extends TestCase
         $this->assertTrue($request->has('foo.baz'));
     }
 
+    public function testWhenHasMethod()
+    {
+        $request = Request::create('/', 'GET', ['name' => 'Taylor', 'age' => '', 'city' => null]);
+
+        $name = $age = $city = $foo = $bar = false;
+
+        $request->whenHas('name', function ($value) use (&$name) {
+            $name = $value;
+        });
+
+        $request->whenHas('age', function ($value) use (&$age) {
+            $age = $value;
+        });
+
+        $request->whenHas('city', function ($value) use (&$city) {
+            $city = $value;
+        });
+
+        $request->whenHas('foo', function () use (&$foo) {
+            $foo = 'test';
+        });
+
+        $request->whenHas('bar', function () use (&$bar) {
+            $bar = 'test';
+        }, function () use (&$bar) {
+            $bar = true;
+        });
+
+        $this->assertSame('Taylor', $name);
+        $this->assertSame('', $age);
+        $this->assertNull($city);
+        $this->assertFalse($foo);
+        $this->assertTrue($bar);
+    }
+
+    public function testWhenFilledMethod()
+    {
+        $request = Request::create('/', 'GET', ['name' => 'Taylor', 'age' => '', 'city' => null]);
+
+        $name = $age = $city = $foo = $bar = false;
+
+        $request->whenFilled('name', function ($value) use (&$name) {
+            $name = $value;
+        });
+
+        $request->whenFilled('age', function ($value) use (&$age) {
+            $age = 'test';
+        });
+
+        $request->whenFilled('city', function ($value) use (&$city) {
+            $city = 'test';
+        });
+
+        $request->whenFilled('foo', function () use (&$foo) {
+            $foo = 'test';
+        });
+
+        $request->whenFilled('bar', function () use (&$bar) {
+            $bar = 'test';
+        }, function () use (&$bar) {
+            $bar = true;
+        });
+
+        $this->assertSame('Taylor', $name);
+        $this->assertFalse($age);
+        $this->assertFalse($city);
+        $this->assertFalse($foo);
+        $this->assertTrue($bar);
+    }
+
     public function testMissingMethod()
     {
         $request = Request::create('/', 'GET', ['name' => 'Taylor', 'age' => '', 'city' => null]);
@@ -360,7 +433,7 @@ class HttpRequestTest extends TestCase
         $this->assertTrue($request->filled('name'));
         $this->assertTrue($request->filled('name', 'email'));
 
-        //test arrays within query string
+        // test arrays within query string
         $request = Request::create('/', 'GET', ['foo' => ['bar', 'baz']]);
         $this->assertTrue($request->filled('foo'));
 
@@ -368,6 +441,23 @@ class HttpRequestTest extends TestCase
         $this->assertTrue($request->filled('foo.bar'));
     }
 
+    public function testIsNotFilledMethod()
+    {
+        $request = Request::create('/', 'GET', ['name' => 'Taylor', 'age' => '', 'city' => null]);
+        $this->assertFalse($request->isNotFilled('name'));
+        $this->assertTrue($request->isNotFilled('age'));
+        $this->assertTrue($request->isNotFilled('city'));
+        $this->assertTrue($request->isNotFilled('foo'));
+        $this->assertFalse($request->isNotFilled(['name', 'email']));
+        $this->assertTrue($request->isNotFilled(['foo', 'age']));
+        $this->assertTrue($request->isNotFilled(['age', 'city']));
+
+        $request = Request::create('/', 'GET', ['foo' => ['bar', 'baz' => '0']]);
+        $this->assertFalse($request->isNotFilled('foo'));
+        $this->assertTrue($request->isNotFilled('foo.bar'));
+        $this->assertFalse($request->isNotFilled('foo.baz'));
+    }
+
     public function testFilledAnyMethod()
     {
         $request = Request::create('/', 'GET', ['name' => 'Taylor', 'age' => '', 'city' => null]);
@@ -415,6 +505,84 @@ class HttpRequestTest extends TestCase
         $this->assertFalse($request->boolean('some_undefined_key'));
     }
 
+    public function testCollectMethod()
+    {
+        $request = Request::create('/', 'GET', ['users' => [1, 2, 3]]);
+
+        $this->assertInstanceOf(Collection::class, $request->collect('users'));
+        $this->assertTrue($request->collect('developers')->isEmpty());
+        $this->assertEquals([1, 2, 3], $request->collect('users')->all());
+        $this->assertEquals(['users' => [1, 2, 3]], $request->collect()->all());
+
+        $request = Request::create('/', 'GET', ['text-payload']);
+        $this->assertEquals(['text-payload'], $request->collect()->all());
+
+        $request = Request::create('/', 'GET', ['email' => 'test@example.com']);
+        $this->assertEquals(['test@example.com'], $request->collect('email')->all());
+
+        $request = Request::create('/', 'GET', []);
+        $this->assertInstanceOf(Collection::class, $request->collect());
+        $this->assertTrue($request->collect()->isEmpty());
+
+        $request = Request::create('/', 'GET', ['users' => [1, 2, 3], 'roles' => [4, 5, 6], 'foo' => ['bar', 'baz'], 'email' => 'test@example.com']);
+        $this->assertInstanceOf(Collection::class, $request->collect(['users']));
+        $this->assertTrue($request->collect(['developers'])->isEmpty());
+        $this->assertTrue($request->collect(['roles'])->isNotEmpty());
+        $this->assertEquals(['roles' => [4, 5, 6]], $request->collect(['roles'])->all());
+        $this->assertEquals(['users' => [1, 2, 3], 'email' => 'test@example.com'], $request->collect(['users', 'email'])->all());
+        $this->assertEquals(collect(['roles' => [4, 5, 6], 'foo' => ['bar', 'baz']]), $request->collect(['roles', 'foo']));
+        $this->assertEquals(['users' => [1, 2, 3], 'roles' => [4, 5, 6], 'foo' => ['bar', 'baz'], 'email' => 'test@example.com'], $request->collect()->all());
+    }
+
+    public function testDateMethod()
+    {
+        $request = Request::create('/', 'GET', [
+            'as_null' => null,
+            'as_invalid' => 'invalid',
+
+            'as_datetime' => '20-01-01 16:30:25',
+            'as_format' => '1577896225',
+            'as_timezone' => '20-01-01 13:30:25',
+
+            'as_date' => '2020-01-01',
+            'as_time' => '16:30:25',
+        ]);
+
+        $current = Carbon::create(2020, 1, 1, 16, 30, 25);
+
+        $this->assertNull($request->date('as_null'));
+        $this->assertNull($request->date('doesnt_exists'));
+
+        $this->assertEquals($current, $request->date('as_datetime'));
+        $this->assertEquals($current, $request->date('as_format', 'U'));
+        $this->assertEquals($current, $request->date('as_timezone', null, 'America/Santiago'));
+
+        $this->assertTrue($request->date('as_date')->isSameDay($current));
+        $this->assertTrue($request->date('as_time')->isSameSecond('16:30:25'));
+    }
+
+    public function testDateMethodExceptionWhenValueInvalid()
+    {
+        $this->expectException(InvalidArgumentException::class);
+
+        $request = Request::create('/', 'GET', [
+            'date' => 'invalid',
+        ]);
+
+        $request->date('date');
+    }
+
+    public function testDateMethodExceptionWhenFormatInvalid()
+    {
+        $this->expectException(InvalidArgumentException::class);
+
+        $request = Request::create('/', 'GET', [
+            'date' => '20-01-01 16:30:25',
+        ]);
+
+        $request->date('date', 'invalid_format');
+    }
+
     public function testArrayAccess()
     {
         $request = Request::create('/', 'GET', ['name' => null, 'foo' => ['bar' => null, 'baz' => '']]);
@@ -428,8 +596,8 @@ class HttpRequestTest extends TestCase
             return $route;
         });
 
-        $this->assertFalse(isset($request['non-existant']));
-        $this->assertNull($request['non-existant']);
+        $this->assertFalse(isset($request['non-existent']));
+        $this->assertNull($request['non-existent']);
 
         $this->assertTrue(isset($request['name']));
         $this->assertNull($request['name']);
@@ -445,6 +613,17 @@ class HttpRequestTest extends TestCase
         $this->assertSame('foo', $request['id']);
     }
 
+    public function testArrayAccessWithoutRouteResolver()
+    {
+        $request = Request::create('/', 'GET', ['name' => 'Taylor']);
+
+        $this->assertFalse(isset($request['non-existent']));
+        $this->assertNull($request['non-existent']);
+
+        $this->assertTrue(isset($request['name']));
+        $this->assertSame('Taylor', $request['name']);
+    }
+
     public function testAllMethod()
     {
         $request = Request::create('/', 'GET', ['name' => 'Taylor', 'age' => null]);
@@ -506,6 +685,14 @@ class HttpRequestTest extends TestCase
         $this->assertSame('Bob', $request->query('foo', 'Bob'));
         $all = $request->query(null);
         $this->assertSame('Taylor', $all['name']);
+
+        $request = Request::create('/', 'GET', ['hello' => 'world', 'user' => ['Taylor', 'Mohamed Said']]);
+        $this->assertSame(['Taylor', 'Mohamed Said'], $request->query('user'));
+        $this->assertSame(['hello' => 'world', 'user' => ['Taylor', 'Mohamed Said']], $request->query->all());
+
+        $request = Request::create('/?hello=world&user[]=Taylor&user[]=Mohamed%20Said', 'GET', []);
+        $this->assertSame(['Taylor', 'Mohamed Said'], $request->query('user'));
+        $this->assertSame(['hello' => 'world', 'user' => ['Taylor', 'Mohamed Said']], $request->query->all());
     }
 
     public function testPostMethod()
@@ -584,6 +771,21 @@ class HttpRequestTest extends TestCase
         $this->assertSame('Dayle', $request->input('buddy'));
     }
 
+    public function testMergeIfMissingMethod()
+    {
+        $request = Request::create('/', 'GET', ['name' => 'Taylor']);
+        $merge = ['boolean_setting' => 0];
+        $request->mergeIfMissing($merge);
+        $this->assertSame('Taylor', $request->input('name'));
+        $this->assertSame(0, $request->input('boolean_setting'));
+
+        $request = Request::create('/', 'GET', ['name' => 'Taylor', 'boolean_setting' => 1]);
+        $merge = ['boolean_setting' => 0];
+        $request->mergeIfMissing($merge);
+        $this->assertSame('Taylor', $request->input('name'));
+        $this->assertSame(1, $request->input('boolean_setting'));
+    }
+
     public function testReplaceMethod()
     {
         $request = Request::create('/', 'GET', ['name' => 'Taylor']);
@@ -608,6 +810,18 @@ class HttpRequestTest extends TestCase
         $this->assertSame('foo', $all['do-this'][0]);
     }
 
+    public function testBearerTokenMethod()
+    {
+        $request = Request::create('/', 'GET', [], [], [], ['HTTP_AUTHORIZATION' => 'Bearer fooBearerbar']);
+        $this->assertSame('fooBearerbar', $request->bearerToken());
+
+        $request = Request::create('/', 'GET', [], [], [], ['HTTP_AUTHORIZATION' => 'Basic foo, Bearer bar']);
+        $this->assertSame('bar', $request->bearerToken());
+
+        $request = Request::create('/', 'GET', [], [], [], ['HTTP_AUTHORIZATION' => 'Bearer foo,bar']);
+        $this->assertSame('foo', $request->bearerToken());
+    }
+
     public function testJSONMethod()
     {
         $payload = ['name' => 'taylor'];
@@ -631,29 +845,41 @@ class HttpRequestTest extends TestCase
         $this->assertEquals($payload, $data);
     }
 
-    public function testPrefers()
+    public function getPrefersCases()
     {
-        $this->assertSame('json', Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'application/json'])->prefers(['json']));
-        $this->assertSame('json', Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'application/json'])->prefers(['html', 'json']));
-        $this->assertSame('application/foo+json', Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'application/foo+json'])->prefers('application/foo+json'));
-        $this->assertSame('json', Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'application/foo+json'])->prefers('json'));
-        $this->assertSame('html', Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'application/json;q=0.5, text/html;q=1.0'])->prefers(['json', 'html']));
-        $this->assertSame('txt', Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'application/json;q=0.5, text/plain;q=1.0, text/html;q=1.0'])->prefers(['json', 'txt', 'html']));
-        $this->assertSame('json', Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'application/*'])->prefers('json'));
-        $this->assertSame('json', Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'application/json; charset=utf-8'])->prefers('json'));
-        $this->assertNull(Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'application/xml; charset=utf-8'])->prefers(['html', 'json']));
-        $this->assertSame('json', Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'application/json, text/html'])->prefers(['html', 'json']));
-        $this->assertSame('html', Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'application/json;q=0.4, text/html;q=0.6'])->prefers(['html', 'json']));
+        return [
+            ['application/json', ['json'], 'json'],
+            ['application/json', ['html', 'json'], 'json'],
+            ['application/foo+json', 'application/foo+json', 'application/foo+json'],
+            ['application/foo+json', 'json', 'json'],
+            ['application/json;q=0.5, text/html;q=1.0', ['json', 'html'], 'html'],
+            ['application/json;q=0.5, text/plain;q=1.0, text/html;q=1.0', ['json', 'txt', 'html'], 'txt'],
+            ['application/*', 'json', 'json'],
+            ['application/json; charset=utf-8', 'json', 'json'],
+            ['application/xml; charset=utf-8', ['html', 'json'], null],
+            ['application/json, text/html', ['html', 'json'], 'json'],
+            ['application/json;q=0.4, text/html;q=0.6', ['html', 'json'], 'html'],
 
-        $this->assertSame('application/json', Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'application/json; charset=utf-8'])->prefers('application/json'));
-        $this->assertSame('application/json', Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'application/json, text/html'])->prefers(['text/html', 'application/json']));
-        $this->assertSame('text/html', Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'application/json;q=0.4, text/html;q=0.6'])->prefers(['text/html', 'application/json']));
-        $this->assertSame('text/html', Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'application/json;q=0.4, text/html;q=0.6'])->prefers(['application/json', 'text/html']));
+            ['application/json; charset=utf-8', 'application/json', 'application/json'],
+            ['application/json, text/html', ['text/html', 'application/json'], 'application/json'],
+            ['application/json;q=0.4, text/html;q=0.6', ['text/html', 'application/json'], 'text/html'],
+            ['application/json;q=0.4, text/html;q=0.6', ['application/json', 'text/html'], 'text/html'],
 
-        $this->assertSame('json', Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => '*/*; charset=utf-8'])->prefers('json'));
-        $this->assertSame('application/json', Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'application/*'])->prefers('application/json'));
-        $this->assertSame('application/xml', Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'application/*'])->prefers('application/xml'));
-        $this->assertNull(Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'application/*'])->prefers('text/html'));
+            ['*/*; charset=utf-8', 'json', 'json'],
+            ['application/*', 'application/json', 'application/json'],
+            ['application/*', 'application/xml', 'application/xml'],
+            ['application/*', 'text/html', null],
+        ];
+    }
+
+    /**
+     * @dataProvider getPrefersCases
+     */
+    public function testPrefersMethod($accept, $prefers, $expected)
+    {
+        $this->assertSame(
+            $expected, Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => $accept])->prefers($prefers)
+        );
     }
 
     public function testAllInputReturnsInputAndFiles()
@@ -726,7 +952,7 @@ class HttpRequestTest extends TestCase
 
     public function testOldMethodCallsSession()
     {
-        $request = Request::create('/', 'GET');
+        $request = Request::create('/');
         $session = m::mock(Store::class);
         $session->shouldReceive('getOldInput')->once()->with('foo', 'bar')->andReturn('boom');
         $request->setLaravelSession($session);
@@ -735,7 +961,7 @@ class HttpRequestTest extends TestCase
 
     public function testFlushMethodCallsSession()
     {
-        $request = Request::create('/', 'GET');
+        $request = Request::create('/');
         $session = m::mock(Store::class);
         $session->shouldReceive('flashInput')->once();
         $request->setLaravelSession($session);
@@ -880,12 +1106,27 @@ class HttpRequestTest extends TestCase
         $this->assertFalse($request->accepts('text/html'));
     }
 
+    public function testCaseInsensitiveAcceptHeader()
+    {
+        $request = Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'APPLICATION/JSON']);
+        $this->assertTrue($request->accepts(['text/html', 'application/json']));
+
+        $request = Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'AppLiCaTion/JsOn']);
+        $this->assertTrue($request->accepts(['text/html', 'application/json']));
+
+        $request = Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'APPLICATION/*']);
+        $this->assertTrue($request->accepts(['text/html', 'application/json']));
+
+        $request = Request::create('/', 'GET', [], [], [], ['HTTP_ACCEPT' => 'APPLICATION/JSON']);
+        $this->assertTrue($request->expectsJson());
+    }
+
     public function testSessionMethod()
     {
         $this->expectException(RuntimeException::class);
         $this->expectExceptionMessage('Session store not set on request.');
 
-        $request = Request::create('/', 'GET');
+        $request = Request::create('/');
         $request->session();
     }
 
@@ -949,19 +1190,19 @@ class HttpRequestTest extends TestCase
         $request = Request::create('/', 'GET', ['foo' => 'bar', 'empty' => '']);
 
         // Parameter 'foo' is 'bar', then it ISSET and is NOT EMPTY.
-        $this->assertEquals($request->foo, 'bar');
-        $this->assertEquals(isset($request->foo), true);
-        $this->assertEquals(empty($request->foo), false);
+        $this->assertSame('bar', $request->foo);
+        $this->assertTrue(isset($request->foo));
+        $this->assertNotEmpty($request->foo);
 
         // Parameter 'empty' is '', then it ISSET and is EMPTY.
-        $this->assertEquals($request->empty, '');
+        $this->assertSame('', $request->empty);
         $this->assertTrue(isset($request->empty));
         $this->assertEmpty($request->empty);
 
         // Parameter 'undefined' is undefined/null, then it NOT ISSET and is EMPTY.
-        $this->assertEquals($request->undefined, null);
-        $this->assertEquals(isset($request->undefined), false);
-        $this->assertEquals(empty($request->undefined), true);
+        $this->assertNull($request->undefined);
+        $this->assertFalse(isset($request->undefined));
+        $this->assertEmpty($request->undefined);
 
         // Simulates Route parameters.
         $request = Request::create('/example/bar', 'GET', ['xyz' => 'overwritten']);
@@ -975,22 +1216,22 @@ class HttpRequestTest extends TestCase
         // Router parameter 'foo' is 'bar', then it ISSET and is NOT EMPTY.
         $this->assertSame('bar', $request->foo);
         $this->assertSame('bar', $request['foo']);
-        $this->assertEquals(isset($request->foo), true);
-        $this->assertEquals(empty($request->foo), false);
+        $this->assertTrue(isset($request->foo));
+        $this->assertNotEmpty($request->foo);
 
         // Router parameter 'undefined' is undefined/null, then it NOT ISSET and is EMPTY.
-        $this->assertEquals($request->undefined, null);
-        $this->assertEquals(isset($request->undefined), false);
-        $this->assertEquals(empty($request->undefined), true);
+        $this->assertNull($request->undefined);
+        $this->assertFalse(isset($request->undefined));
+        $this->assertEmpty($request->undefined);
 
         // Special case: router parameter 'xyz' is 'overwritten' by QueryString, then it ISSET and is NOT EMPTY.
         // Basically, QueryStrings have priority over router parameters.
-        $this->assertEquals($request->xyz, 'overwritten');
-        $this->assertEquals(isset($request->foo), true);
-        $this->assertEquals(empty($request->foo), false);
+        $this->assertSame('overwritten', $request->xyz);
+        $this->assertTrue(isset($request->foo));
+        $this->assertNotEmpty($request->foo);
 
         // Simulates empty QueryString and Routes.
-        $request = Request::create('/', 'GET');
+        $request = Request::create('/');
         $request->setRouteResolver(function () use ($request) {
             $route = new Route('GET', '/', []);
             $route->bind($request);
@@ -999,18 +1240,18 @@ class HttpRequestTest extends TestCase
         });
 
         // Parameter 'undefined' is undefined/null, then it NOT ISSET and is EMPTY.
-        $this->assertEquals($request->undefined, null);
-        $this->assertEquals(isset($request->undefined), false);
-        $this->assertEquals(empty($request->undefined), true);
+        $this->assertNull($request->undefined);
+        $this->assertFalse(isset($request->undefined));
+        $this->assertEmpty($request->undefined);
 
         // Special case: simulates empty QueryString and Routes, without the Route Resolver.
         // It'll happen when you try to get a parameter outside a route.
-        $request = Request::create('/', 'GET');
+        $request = Request::create('/');
 
         // Parameter 'undefined' is undefined/null, then it NOT ISSET and is EMPTY.
-        $this->assertEquals($request->undefined, null);
-        $this->assertEquals(isset($request->undefined), false);
-        $this->assertEquals(empty($request->undefined), true);
+        $this->assertNull($request->undefined);
+        $this->assertFalse(isset($request->undefined));
+        $this->assertEmpty($request->undefined);
     }
 
     public function testHttpRequestFlashCallsSessionFlashInputWithInputData()
