@@ -2,22 +2,39 @@
 
 namespace Illuminate\Tests\Integration\Auth;
 
+use Illuminate\Auth\Access\Events\GateEvaluated;
+use Illuminate\Support\Facades\Event;
 use Illuminate\Support\Facades\Gate;
 use Illuminate\Tests\Integration\Auth\Fixtures\AuthenticationTestUser;
 use Illuminate\Tests\Integration\Auth\Fixtures\Policies\AuthenticationTestUserPolicy;
 use Orchestra\Testbench\TestCase;
 
-/**
- * @group integration
- */
 class GatePolicyResolutionTest extends TestCase
 {
+    public function testGateEvaluationEventIsFired()
+    {
+        Event::fake();
+
+        Gate::check('foo');
+
+        Event::assertDispatched(GateEvaluated::class);
+    }
+
     public function testPolicyCanBeGuessedUsingClassConventions()
     {
         $this->assertInstanceOf(
             AuthenticationTestUserPolicy::class,
             Gate::getPolicyFor(AuthenticationTestUser::class)
         );
+
+        $this->assertInstanceOf(
+            AuthenticationTestUserPolicy::class,
+            Gate::getPolicyFor(Fixtures\Models\AuthenticationTestUser::class)
+        );
+
+        $this->assertNull(
+            Gate::getPolicyFor(static::class)
+        );
     }
 
     public function testPolicyCanBeGuessedUsingCallback()
