@@ -13,15 +13,10 @@ use Illuminate\Support\Facades\Schema;
 use Illuminate\Support\Str;
 use Illuminate\Tests\Integration\Database\DatabaseTestCase;
 
-/**
- * @group integration
- */
 class EloquentBelongsToManyTest extends DatabaseTestCase
 {
-    protected function setUp(): void
+    protected function defineDatabaseMigrationsAfterDatabaseRefreshed()
     {
-        parent::setUp();
-
         Schema::create('users', function (Blueprint $table) {
             $table->increments('id');
             $table->string('uuid');
@@ -43,6 +38,7 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         });
 
         Schema::create('users_posts', function (Blueprint $table) {
+            $table->increments('id');
             $table->string('user_uuid');
             $table->string('post_uuid');
             $table->tinyInteger('is_draft')->default(1);
@@ -51,12 +47,11 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
 
         Schema::create('posts_tags', function (Blueprint $table) {
             $table->integer('post_id');
-            $table->integer('tag_id');
-            $table->string('flag')->default('');
+            $table->integer('tag_id')->default(0);
+            $table->string('tag_name')->default('')->nullable();
+            $table->string('flag')->default('')->nullable();
             $table->timestamps();
         });
-
-        Carbon::setTestNow(null);
     }
 
     public function testBasicCreateAndRetrieve()
@@ -90,7 +85,7 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertEquals(
             [
                 'post_id' => '1', 'tag_id' => '1', 'flag' => 'taylor',
-                'created_at' => '2017-10-10 10:10:10', 'updated_at' => '2017-10-10 10:10:10',
+                'created_at' => '2017-10-10T10:10:10.000000Z', 'updated_at' => '2017-10-10T10:10:10.000000Z',
             ],
             $post->tags[0]->pivot->toArray()
         );
@@ -135,7 +130,7 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $post->tagsWithCustomPivot()->attach($tag->id);
 
         $this->assertInstanceOf(PostTagPivot::class, $post->tagsWithCustomPivot[0]->pivot);
-        $this->assertSame('1507630210', $post->tagsWithCustomPivot[0]->pivot->getAttributes()['created_at']);
+        $this->assertEquals('1507630210', $post->tagsWithCustomPivot[0]->pivot->created_at);
 
         $this->assertInstanceOf(PostTagPivot::class, $post->tagsWithCustomPivotClass[0]->pivot);
         $this->assertSame('posts_tags', $post->tagsWithCustomPivotClass()->getTable());
@@ -206,6 +201,7 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         );
     }
 
+    /** @group SkipMSSQL */
     public function testCustomPivotClassUpdatesTimestamps()
     {
         Carbon::setTestNow('2017-10-10 10:10:10');
@@ -216,8 +212,8 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         DB::table('posts_tags')->insert([
             [
                 'post_id' => $post->id, 'tag_id' => $tag->id, 'flag' => 'empty',
-                'created_at' => '1507630210',
-                'updated_at' => '1507630210',
+                'created_at' => '2017-10-10 10:10:10',
+                'updated_at' => '2017-10-10 10:10:10',
             ],
         ]);
 
@@ -229,8 +225,8 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         );
         foreach ($post->tagsWithCustomExtraPivot as $tag) {
             $this->assertSame('exclude', $tag->pivot->flag);
-            $this->assertEquals('1507630210', $tag->pivot->getAttributes()['created_at']);
-            $this->assertEquals('1507630220', $tag->pivot->getAttributes()['updated_at']); // +10 seconds
+            $this->assertEquals('2017-10-10 10:10:10', $tag->pivot->getAttributes()['created_at']);
+            $this->assertEquals('2017-10-10 10:10:20', $tag->pivot->getAttributes()['updated_at']); // +10 seconds
         }
     }
 
@@ -334,7 +330,7 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
 
         $post = Post::create(['title' => Str::random()]);
 
-        $post->tags()->firstOrFail(['id' => 10]);
+        $post->tags()->firstOrFail(['id']);
     }
 
     public function testFindMethod()
@@ -347,12 +343,16 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $post->tags()->attach(Tag::all());
 
         $this->assertEquals($tag2->name, $post->tags()->find($tag2->id)->name);
+        $this->assertCount(0, $post->tags()->findMany([]));
         $this->assertCount(2, $post->tags()->findMany([$tag->id, $tag2->id]));
+        $this->assertCount(0, $post->tags()->findMany(new Collection));
+        $this->assertCount(2, $post->tags()->findMany(new Collection([$tag->id, $tag2->id])));
     }
 
     public function testFindOrFailMethod()
     {
         $this->expectException(ModelNotFoundException::class);
+        $this->expectExceptionMessage('No query results for model [Illuminate\Tests\Integration\Database\EloquentBelongsToManyTest\Tag] 10');
 
         $post = Post::create(['title' => Str::random()]);
 
@@ -363,6 +363,34 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $post->tags()->findOrFail(10);
     }
 
+    public function testFindOrFailMethodWithMany()
+    {
+        $this->expectException(ModelNotFoundException::class);
+        $this->expectExceptionMessage('No query results for model [Illuminate\Tests\Integration\Database\EloquentBelongsToManyTest\Tag] 10, 11');
+
+        $post = Post::create(['title' => Str::random()]);
+
+        Tag::create(['name' => Str::random()]);
+
+        $post->tags()->attach(Tag::all());
+
+        $post->tags()->findOrFail([10, 11]);
+    }
+
+    public function testFindOrFailMethodWithManyUsingCollection()
+    {
+        $this->expectException(ModelNotFoundException::class);
+        $this->expectExceptionMessage('No query results for model [Illuminate\Tests\Integration\Database\EloquentBelongsToManyTest\Tag] 10, 11');
+
+        $post = Post::create(['title' => Str::random()]);
+
+        Tag::create(['name' => Str::random()]);
+
+        $post->tags()->attach(Tag::all());
+
+        $post->tags()->findOrFail(new Collection([10, 11]));
+    }
+
     public function testFindOrNewMethod()
     {
         $post = Post::create(['title' => Str::random()]);
@@ -373,8 +401,8 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
 
         $this->assertEquals($tag->id, $post->tags()->findOrNew($tag->id)->id);
 
-        $this->assertNull($post->tags()->findOrNew('asd')->id);
-        $this->assertInstanceOf(Tag::class, $post->tags()->findOrNew('asd'));
+        $this->assertNull($post->tags()->findOrNew(666)->id);
+        $this->assertInstanceOf(Tag::class, $post->tags()->findOrNew(666));
     }
 
     public function testFirstOrNewMethod()
@@ -387,8 +415,8 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
 
         $this->assertEquals($tag->id, $post->tags()->firstOrNew(['id' => $tag->id])->id);
 
-        $this->assertNull($post->tags()->firstOrNew(['id' => 'asd'])->id);
-        $this->assertInstanceOf(Tag::class, $post->tags()->firstOrNew(['id' => 'asd']));
+        $this->assertNull($post->tags()->firstOrNew(['id' => 666])->id);
+        $this->assertInstanceOf(Tag::class, $post->tags()->firstOrNew(['id' => 666]));
     }
 
     public function testFirstOrCreateMethod()
@@ -417,7 +445,7 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $post->tags()->updateOrCreate(['id' => $tag->id], ['name' => 'wavez']);
         $this->assertSame('wavez', $tag->fresh()->name);
 
-        $post->tags()->updateOrCreate(['id' => 'asd'], ['name' => 'dives']);
+        $post->tags()->updateOrCreate(['id' => 666], ['name' => 'dives']);
         $this->assertNotNull($post->tags()->whereName('dives')->first());
     }
 
@@ -569,6 +597,7 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertNotSame('2017-10-10 10:10:10', $tag->fresh()->updated_at->toDateTimeString());
     }
 
+    /** @group SkipMSSQL */
     public function testCanRetrieveRelatedIds()
     {
         $post = Post::create(['title' => Str::random()]);
@@ -587,6 +616,7 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertEquals([200, 400], $post->tags()->allRelatedIds()->toArray());
     }
 
+    /** @group SkipMSSQL */
     public function testCanTouchRelatedModels()
     {
         $post = Post::create(['title' => Str::random()]);
@@ -613,6 +643,7 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertNotSame('2017-10-10 10:10:10', Tag::find(300)->updated_at);
     }
 
+    /** @group SkipMSSQL */
     public function testWherePivotOnString()
     {
         $tag = Tag::create(['name' => Str::random()]);
@@ -629,6 +660,7 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertEquals($relationTag->getAttributes(), $tag->getAttributes());
     }
 
+    /** @group SkipMSSQL */
     public function testFirstWhere()
     {
         $tag = Tag::create(['name' => 'foo']);
@@ -645,6 +677,7 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertEquals($relationTag->getAttributes(), $tag->getAttributes());
     }
 
+    /** @group SkipMSSQL */
     public function testWherePivotOnBoolean()
     {
         $tag = Tag::create(['name' => Str::random()]);
@@ -661,6 +694,7 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertEquals($relationTag->getAttributes(), $tag->getAttributes());
     }
 
+    /** @group SkipMSSQL */
     public function testWherePivotInMethod()
     {
         $tag = Tag::create(['name' => Str::random()]);
@@ -695,6 +729,7 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertEquals($relationTags->pluck('id')->toArray(), [$tag1->id, $tag3->id]);
     }
 
+    /** @group SkipMSSQL */
     public function testWherePivotNotInMethod()
     {
         $tag1 = Tag::create(['name' => Str::random()]);
@@ -733,6 +768,42 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertEquals($relationTags->pluck('id')->toArray(), [$tag1->id, $tag2->id]);
     }
 
+    /** @group SkipMSSQL */
+    public function testWherePivotNullMethod()
+    {
+        $tag1 = Tag::create(['name' => Str::random()]);
+        $tag2 = Tag::create(['name' => Str::random()]);
+        $post = Post::create(['title' => Str::random()]);
+
+        DB::table('posts_tags')->insert([
+            ['post_id' => $post->id, 'tag_id' => $tag1->id, 'flag' => 'foo'],
+        ]);
+        DB::table('posts_tags')->insert([
+            ['post_id' => $post->id, 'tag_id' => $tag2->id, 'flag' => null],
+        ]);
+
+        $relationTag = $post->tagsWithExtraPivot()->wherePivotNull('flag')->first();
+        $this->assertEquals($relationTag->getAttributes(), $tag2->getAttributes());
+    }
+
+    /** @group SkipMSSQL */
+    public function testWherePivotNotNullMethod()
+    {
+        $tag1 = Tag::create(['name' => Str::random()]);
+        $tag2 = Tag::create(['name' => Str::random()]);
+        $post = Post::create(['title' => Str::random()]);
+
+        DB::table('posts_tags')->insert([
+            ['post_id' => $post->id, 'tag_id' => $tag1->id, 'flag' => 'foo'],
+        ]);
+        DB::table('posts_tags')->insert([
+            ['post_id' => $post->id, 'tag_id' => $tag2->id, 'flag' => null],
+        ]);
+
+        $relationTag = $post->tagsWithExtraPivot()->wherePivotNotNull('flag')->first();
+        $this->assertEquals($relationTag->getAttributes(), $tag1->getAttributes());
+    }
+
     public function testCanUpdateExistingPivot()
     {
         $tag = Tag::create(['name' => Str::random()]);
@@ -790,17 +861,17 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $post = Post::create(['title' => Str::random()]);
 
         $tag = $post->tagsWithCustomRelatedKey()->create(['name' => Str::random()]);
-        $this->assertEquals($tag->name, $post->tagsWithCustomRelatedKey()->first()->pivot->tag_id);
+        $this->assertEquals($tag->name, $post->tagsWithCustomRelatedKey()->first()->pivot->tag_name);
 
         $post->tagsWithCustomRelatedKey()->detach($tag);
 
         $post->tagsWithCustomRelatedKey()->attach($tag);
-        $this->assertEquals($tag->name, $post->tagsWithCustomRelatedKey()->first()->pivot->tag_id);
+        $this->assertEquals($tag->name, $post->tagsWithCustomRelatedKey()->first()->pivot->tag_name);
 
         $post->tagsWithCustomRelatedKey()->detach(new Collection([$tag]));
 
         $post->tagsWithCustomRelatedKey()->attach(new Collection([$tag]));
-        $this->assertEquals($tag->name, $post->tagsWithCustomRelatedKey()->first()->pivot->tag_id);
+        $this->assertEquals($tag->name, $post->tagsWithCustomRelatedKey()->first()->pivot->tag_name);
 
         $post->tagsWithCustomRelatedKey()->updateExistingPivot($tag, ['flag' => 'exclude']);
         $this->assertSame('exclude', $post->tagsWithCustomRelatedKey()->first()->pivot->flag);
@@ -839,22 +910,80 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $user->postsWithCustomPivot()->updateExistingPivot($post2->uuid, ['is_draft' => 0]);
         $this->assertEquals(0, $user->postsWithCustomPivot()->first()->pivot->is_draft);
     }
+
+    /** @group SkipMSSQL */
+    public function testOrderByPivotMethod()
+    {
+        $tag1 = Tag::create(['name' => Str::random()]);
+        $tag2 = Tag::create(['name' => Str::random()]);
+        $tag3 = Tag::create(['name' => Str::random()]);
+        $tag4 = Tag::create(['name' => Str::random()]);
+        $post = Post::create(['title' => Str::random()]);
+
+        DB::table('posts_tags')->insert([
+            ['post_id' => $post->id, 'tag_id' => $tag1->id, 'flag' => 'foo3'],
+            ['post_id' => $post->id, 'tag_id' => $tag2->id, 'flag' => 'foo1'],
+            ['post_id' => $post->id, 'tag_id' => $tag3->id, 'flag' => 'foo4'],
+            ['post_id' => $post->id, 'tag_id' => $tag4->id, 'flag' => 'foo2'],
+        ]);
+
+        $relationTag1 = $post->tagsWithCustomExtraPivot()->orderByPivot('flag', 'asc')->first();
+        $this->assertEquals($relationTag1->getAttributes(), $tag2->getAttributes());
+
+        $relationTag2 = $post->tagsWithCustomExtraPivot()->orderByPivot('flag', 'desc')->first();
+        $this->assertEquals($relationTag2->getAttributes(), $tag3->getAttributes());
+    }
+
+    public function testFirstOrMethod()
+    {
+        $user1 = User::create(['name' => Str::random()]);
+        $user2 = User::create(['name' => Str::random()]);
+        $user3 = User::create(['name' => Str::random()]);
+        $post1 = Post::create(['title' => Str::random()]);
+        $post2 = Post::create(['title' => Str::random()]);
+        $post3 = Post::create(['title' => Str::random()]);
+
+        $user1->posts()->sync([$post1->uuid, $post2->uuid]);
+        $user2->posts()->sync([$post1->uuid, $post2->uuid]);
+
+        $this->assertEquals(
+            $post1->id,
+            $user2->posts()->firstOr(function () {
+                return Post::create(['title' => Str::random()]);
+            })->id
+        );
+
+        $this->assertEquals(
+            $post3->id,
+            $user3->posts()->firstOr(function () use ($post3) {
+                return $post3;
+            })->id
+        );
+    }
 }
 
 class User extends Model
 {
     public $table = 'users';
     public $timestamps = true;
-    protected $guarded = ['id'];
+    protected $guarded = [];
 
     protected static function boot()
     {
         parent::boot();
+
         static::creating(function ($model) {
             $model->setAttribute('uuid', Str::random());
         });
     }
 
+    public function posts()
+    {
+        return $this->belongsToMany(Post::class, 'users_posts', 'user_uuid', 'post_uuid', 'uuid', 'uuid')
+            ->withPivot('is_draft')
+            ->withTimestamps();
+    }
+
     public function postsWithCustomPivot()
     {
         return $this->belongsToMany(Post::class, 'users_posts', 'user_uuid', 'post_uuid', 'uuid', 'uuid')
@@ -868,17 +997,25 @@ class Post extends Model
 {
     public $table = 'posts';
     public $timestamps = true;
-    protected $guarded = ['id'];
+    protected $guarded = [];
     protected $touches = ['touchingTags'];
 
     protected static function boot()
     {
         parent::boot();
+
         static::creating(function ($model) {
             $model->setAttribute('uuid', Str::random());
         });
     }
 
+    public function users()
+    {
+        return $this->belongsToMany(User::class, 'users_posts', 'post_uuid', 'user_uuid', 'uuid', 'uuid')
+            ->withPivot('is_draft')
+            ->withTimestamps();
+    }
+
     public function tags()
     {
         return $this->belongsToMany(Tag::class, 'posts_tags', 'post_id', 'tag_id')
@@ -928,7 +1065,7 @@ class Post extends Model
 
     public function tagsWithCustomRelatedKey()
     {
-        return $this->belongsToMany(Tag::class, 'posts_tags', 'post_id', 'tag_id', 'id', 'name')
+        return $this->belongsToMany(Tag::class, 'posts_tags', 'post_id', 'tag_name', 'id', 'name')
             ->withPivot('flag');
     }
 
@@ -942,7 +1079,7 @@ class Tag extends Model
 {
     public $table = 'tags';
     public $timestamps = true;
-    protected $guarded = ['id'];
+    protected $fillable = ['name'];
 
     public function posts()
     {
@@ -954,7 +1091,7 @@ class TouchingTag extends Model
 {
     public $table = 'tags';
     public $timestamps = true;
-    protected $guarded = ['id'];
+    protected $guarded = [];
     protected $touches = ['posts'];
 
     public function posts()
@@ -967,7 +1104,7 @@ class TagWithCustomPivot extends Model
 {
     public $table = 'tags';
     public $timestamps = true;
-    protected $guarded = ['id'];
+    protected $guarded = [];
 
     public function posts()
     {
@@ -983,14 +1120,18 @@ class UserPostPivot extends Pivot
 class PostTagPivot extends Pivot
 {
     protected $table = 'posts_tags';
-    protected $dateFormat = 'U';
+
+    public function getCreatedAtAttribute($value)
+    {
+        return Carbon::parse($value)->format('U');
+    }
 }
 
 class TagWithGlobalScope extends Model
 {
     public $table = 'tags';
     public $timestamps = true;
-    protected $guarded = ['id'];
+    protected $guarded = [];
 
     public static function boot()
     {
