@@ -2,19 +2,15 @@
 
 namespace Illuminate\Tests\Integration\Database;
 
+use Illuminate\Database\Eloquent\Collection as EloquentCollection;
 use Illuminate\Database\Schema\Blueprint;
 use Illuminate\Support\Facades\Schema;
 use Illuminate\Tests\Integration\Database\Fixtures\User;
 
-/**
- * @group integration
- */
 class EloquentCollectionFreshTest extends DatabaseTestCase
 {
-    protected function setUp(): void
+    protected function defineDatabaseMigrationsAfterDatabaseRefreshed()
     {
-        parent::setUp();
-
         Schema::create('users', function (Blueprint $table) {
             $table->increments('id');
             $table->string('email');
@@ -30,8 +26,11 @@ class EloquentCollectionFreshTest extends DatabaseTestCase
 
         $collection = User::all();
 
-        User::whereKey($collection->pluck('id')->toArray())->delete();
+        $collection->first()->delete();
 
-        $this->assertEmpty($collection->fresh()->filter());
+        $freshCollection = $collection->fresh();
+
+        $this->assertCount(1, $freshCollection);
+        $this->assertInstanceOf(EloquentCollection::class, $freshCollection);
     }
 }
