@@ -8,24 +8,24 @@ use Illuminate\Support\Facades\Event;
 use Illuminate\Support\Facades\Schema;
 use Illuminate\Tests\Integration\Database\DatabaseTestCase;
 
-/**
- * @group integration
- */
 class EloquentModelCustomEventsTest extends DatabaseTestCase
 {
     protected function setUp(): void
     {
         parent::setUp();
 
-        Schema::create('test_model1', function (Blueprint $table) {
-            $table->increments('id');
-        });
-
         Event::listen(CustomEvent::class, function () {
             $_SERVER['fired_event'] = true;
         });
     }
 
+    protected function defineDatabaseMigrationsAfterDatabaseRefreshed()
+    {
+        Schema::create('test_model1', function (Blueprint $table) {
+            $table->increments('id');
+        });
+    }
+
     public function testFlushListenersClearsCustomEvents()
     {
         $_SERVER['fired_event'] = false;
@@ -52,7 +52,7 @@ class TestModel1 extends Model
     public $dispatchesEvents = ['created' => CustomEvent::class];
     public $table = 'test_model1';
     public $timestamps = false;
-    protected $guarded = ['id'];
+    protected $guarded = [];
 }
 
 class CustomEvent
