@@ -6,15 +6,10 @@ use Illuminate\Database\Eloquent\Model;
 use Illuminate\Database\Schema\Blueprint;
 use Illuminate\Support\Facades\Schema;
 
-/**
- * @group integration
- */
 class EloquentModelWithoutEventsTest extends DatabaseTestCase
 {
-    protected function setUp(): void
+    protected function defineDatabaseMigrationsAfterDatabaseRefreshed()
     {
-        parent::setUp();
-
         Schema::create('auto_filled_models', function (Blueprint $table) {
             $table->increments('id');
             $table->text('project')->nullable();
@@ -31,7 +26,7 @@ class EloquentModelWithoutEventsTest extends DatabaseTestCase
 
         $model->save();
 
-        $this->assertEquals('Laravel', $model->project);
+        $this->assertSame('Laravel', $model->project);
     }
 }
 
@@ -39,7 +34,7 @@ class AutoFilledModel extends Model
 {
     public $table = 'auto_filled_models';
     public $timestamps = false;
-    protected $guarded = ['id'];
+    protected $guarded = [];
 
     public static function boot()
     {
