@@ -8,6 +8,7 @@ use Illuminate\Tests\Integration\Foundation\Fixtures\EventDiscovery\Events\Event
 use Illuminate\Tests\Integration\Foundation\Fixtures\EventDiscovery\Listeners\AbstractListener;
 use Illuminate\Tests\Integration\Foundation\Fixtures\EventDiscovery\Listeners\Listener;
 use Illuminate\Tests\Integration\Foundation\Fixtures\EventDiscovery\Listeners\ListenerInterface;
+use Illuminate\Tests\Integration\Foundation\Fixtures\EventDiscovery\UnionListeners\UnionListener;
 use Orchestra\Testbench\TestCase;
 
 class DiscoverEventsTest extends TestCase
@@ -30,4 +31,23 @@ class DiscoverEventsTest extends TestCase
             ],
         ], $events);
     }
+
+    /**
+     * @requires PHP >= 8
+     */
+    public function testUnionEventsCanBeDiscovered()
+    {
+        class_alias(UnionListener::class, 'Tests\Integration\Foundation\Fixtures\EventDiscovery\UnionListeners\UnionListener');
+
+        $events = DiscoverEvents::within(__DIR__.'/Fixtures/EventDiscovery/UnionListeners', getcwd());
+
+        $this->assertEquals([
+            EventOne::class => [
+                UnionListener::class.'@handle',
+            ],
+            EventTwo::class => [
+                UnionListener::class.'@handle',
+            ],
+        ], $events);
+    }
 }
