@@ -2,10 +2,15 @@
 
 namespace Illuminate\Tests\Integration\Http;
 
+use Illuminate\Foundation\Http\Middleware\ValidatePostSize;
+use Illuminate\Http\Exceptions\PostTooLargeException;
+use Illuminate\Http\Request;
 use Illuminate\Http\Resources\ConditionallyLoadsAttributes;
 use Illuminate\Http\Resources\Json\JsonResource;
 use Illuminate\Http\Resources\MergeValue;
 use Illuminate\Http\Resources\MissingValue;
+use Illuminate\Pagination\Cursor;
+use Illuminate\Pagination\CursorPaginator;
 use Illuminate\Pagination\LengthAwarePaginator;
 use Illuminate\Support\Collection;
 use Illuminate\Support\Facades\Route;
@@ -15,8 +20,13 @@ use Illuminate\Tests\Integration\Http\Fixtures\EmptyPostCollectionResource;
 use Illuminate\Tests\Integration\Http\Fixtures\ObjectResource;
 use Illuminate\Tests\Integration\Http\Fixtures\Post;
 use Illuminate\Tests\Integration\Http\Fixtures\PostCollectionResource;
+use Illuminate\Tests\Integration\Http\Fixtures\PostCollectionResourceWithPaginationInformation;
 use Illuminate\Tests\Integration\Http\Fixtures\PostResource;
+use Illuminate\Tests\Integration\Http\Fixtures\PostResourceWithAnonymousResourceCollectionWithPaginationInformation;
 use Illuminate\Tests\Integration\Http\Fixtures\PostResourceWithExtraData;
+use Illuminate\Tests\Integration\Http\Fixtures\PostResourceWithJsonOptions;
+use Illuminate\Tests\Integration\Http\Fixtures\PostResourceWithJsonOptionsAndTypeHints;
+use Illuminate\Tests\Integration\Http\Fixtures\PostResourceWithOptionalAppendedAttributes;
 use Illuminate\Tests\Integration\Http\Fixtures\PostResourceWithOptionalData;
 use Illuminate\Tests\Integration\Http\Fixtures\PostResourceWithOptionalMerging;
 use Illuminate\Tests\Integration\Http\Fixtures\PostResourceWithOptionalPivotRelationship;
@@ -26,11 +36,9 @@ use Illuminate\Tests\Integration\Http\Fixtures\ReallyEmptyPostResource;
 use Illuminate\Tests\Integration\Http\Fixtures\ResourceWithPreservedKeys;
 use Illuminate\Tests\Integration\Http\Fixtures\SerializablePostResource;
 use Illuminate\Tests\Integration\Http\Fixtures\Subscription;
+use Mockery;
 use Orchestra\Testbench\TestCase;
 
-/**
- * @group integration
- */
 class ResourceTest extends TestCase
 {
     public function testResourcesMayBeConvertedToJson()
@@ -39,6 +47,7 @@ class ResourceTest extends TestCase
             return new PostResource(new Post([
                 'id' => 5,
                 'title' => 'Test Title',
+                'abstract' => 'Test abstract',
             ]));
         });
 
@@ -56,6 +65,17 @@ class ResourceTest extends TestCase
         ]);
     }
 
+    public function testResourcesMayBeConvertedToJsonWithToJsonMethod()
+    {
+        $resource = new PostResource(new Post([
+            'id' => 5,
+            'title' => 'Test Title',
+            'abstract' => 'Test abstract',
+        ]));
+
+        $this->assertSame('{"id":5,"title":"Test Title","custom":true}', $resource->toJson());
+    }
+
     public function testAnObjectsMayBeConvertedToJson()
     {
         Route::get('/', function () {
@@ -141,6 +161,59 @@ class ResourceTest extends TestCase
         ]);
     }
 
+    public function testResourcesMayHaveOptionalAppendedAttributes()
+    {
+        Route::get('/', function () {
+            $post = new Post([
+                'id' => 5,
+            ]);
+
+            $post->append('is_published');
+
+            return new PostResourceWithOptionalAppendedAttributes($post);
+        });
+
+        $response = $this->withoutExceptionHandling()->get(
+            '/', ['Accept' => 'application/json']
+        );
+
+        $response->assertStatus(200);
+
+        $response->assertJson([
+            'data' => [
+                'id' => 5,
+                'first' => true,
+                'second' => 'override value',
+                'third' => 'override value',
+                'fourth' => true,
+                'fifth' => true,
+            ],
+        ]);
+    }
+
+    public function testResourcesWithOptionalAppendedAttributesReturnDefaultValuesAndNotMissingValues()
+    {
+        Route::get('/', function () {
+            return new PostResourceWithOptionalAppendedAttributes(new Post([
+                'id' => 5,
+            ]));
+        });
+
+        $response = $this->withoutExceptionHandling()->get(
+            '/', ['Accept' => 'application/json']
+        );
+
+        $response->assertStatus(200);
+
+        $response->assertExactJson([
+            'data' => [
+                'id' => 5,
+                'fourth' => 'default',
+                'fifth' => 'default',
+            ],
+        ]);
+    }
+
     public function testResourcesMayHaveOptionalMerges()
     {
         Route::get('/', function () {
@@ -424,6 +497,85 @@ class ResourceTest extends TestCase
         ]);
     }
 
+    public function testResourcesMayCustomizeJsonOptions()
+    {
+        Route::get('/', function () {
+            return new PostResourceWithJsonOptions(new Post([
+                'id' => 5,
+                'title' => 'Test Title',
+                'reading_time' => 3.0,
+            ]));
+        });
+
+        $response = $this->withoutExceptionHandling()->get(
+            '/', ['Accept' => 'application/json']
+        );
+
+        $this->assertEquals(
+            '{"data":{"id":5,"title":"Test Title","reading_time":3.0}}',
+            $response->baseResponse->content()
+        );
+    }
+
+    public function testCollectionResourcesMayCustomizeJsonOptions()
+    {
+        Route::get('/', function () {
+            return PostResourceWithJsonOptions::collection(collect([
+                new Post(['id' => 5, 'title' => 'Test Title', 'reading_time' => 3.0]),
+            ]));
+        });
+
+        $response = $this->withoutExceptionHandling()->get(
+            '/', ['Accept' => 'application/json']
+        );
+
+        $this->assertEquals(
+            '{"data":[{"id":5,"title":"Test Title","reading_time":3.0}]}',
+            $response->baseResponse->content()
+        );
+    }
+
+    public function testResourcesMayCustomizeJsonOptionsOnPaginatedResponse()
+    {
+        Route::get('/', function () {
+            $paginator = new LengthAwarePaginator(
+                collect([new Post(['id' => 5, 'title' => 'Test Title', 'reading_time' => 3.0])]),
+                10, 15, 1
+            );
+
+            return PostResourceWithJsonOptions::collection($paginator);
+        });
+
+        $response = $this->withoutExceptionHandling()->get(
+            '/', ['Accept' => 'application/json']
+        );
+
+        $this->assertEquals(
+            '{"data":[{"id":5,"title":"Test Title","reading_time":3.0}],"links":{"first":"\/?page=1","last":"\/?page=1","prev":null,"next":null},"meta":{"current_page":1,"from":1,"last_page":1,"links":[{"url":null,"label":"&laquo; Previous","active":false},{"url":"\/?page=1","label":"1","active":true},{"url":null,"label":"Next &raquo;","active":false}],"path":"\/","per_page":15,"to":1,"total":10}}',
+            $response->baseResponse->content()
+        );
+    }
+
+    public function testResourcesMayCustomizeJsonOptionsWithTypeHintedConstructor()
+    {
+        Route::get('/', function () {
+            return new PostResourceWithJsonOptionsAndTypeHints(new Post([
+                'id' => 5,
+                'title' => 'Test Title',
+                'reading_time' => 3.0,
+            ]));
+        });
+
+        $response = $this->withoutExceptionHandling()->get(
+            '/', ['Accept' => 'application/json']
+        );
+
+        $this->assertEquals(
+            '{"data":{"id":5,"title":"Test Title","reading_time":3.0}}',
+            $response->baseResponse->content()
+        );
+    }
+
     public function testCustomHeadersMayBeSetOnResponses()
     {
         Route::get('/', function () {
@@ -612,6 +764,117 @@ class ResourceTest extends TestCase
         ]);
     }
 
+    public function testCursorPaginatorReceiveLinks()
+    {
+        Route::get('/', function () {
+            $paginator = new CursorPaginator(
+                collect([new Post(['id' => 5, 'title' => 'Test Title']), new Post(['id' => 6, 'title' => 'Hello'])]),
+                1, null, ['parameters' => ['id']]
+            );
+
+            return new PostCollectionResource($paginator);
+        });
+
+        $response = $this->withoutExceptionHandling()->get(
+            '/', ['Accept' => 'application/json']
+        );
+
+        $response->assertStatus(200);
+
+        $response->assertJson([
+            'data' => [
+                [
+                    'id' => 5,
+                    'title' => 'Test Title',
+                ],
+            ],
+            'links' => [
+                'first' => null,
+                'last' => null,
+                'prev' => null,
+                'next' => '/?cursor='.(new Cursor(['id' => 5]))->encode(),
+            ],
+            'meta' => [
+                'path' => '/',
+                'per_page' => 1,
+            ],
+        ]);
+    }
+
+    public function testCursorPaginatorResourceCanPreserveQueryParameters()
+    {
+        Route::get('/', function () {
+            $collection = collect([new Post(['id' => 5, 'title' => 'Test Title']), new Post(['id' => 6, 'title' => 'Hello'])]);
+            $paginator = new CursorPaginator(
+                $collection, 1, null, ['parameters' => ['id']]
+            );
+
+            return PostCollectionResource::make($paginator)->preserveQuery();
+        });
+
+        $response = $this->withoutExceptionHandling()->get(
+            '/?framework=laravel&author=Otwell', ['Accept' => 'application/json']
+        );
+
+        $response->assertStatus(200);
+
+        $response->assertJson([
+            'data' => [
+                [
+                    'id' => 5,
+                    'title' => 'Test Title',
+                ],
+            ],
+            'links' => [
+                'first' => null,
+                'last' => null,
+                'prev' => null,
+                'next' => '/?framework=laravel&author=Otwell&cursor='.(new Cursor(['id' => 5]))->encode(),
+            ],
+            'meta' => [
+                'path' => '/',
+                'per_page' => 1,
+            ],
+        ]);
+    }
+
+    public function testCursorPaginatorResourceCanReceiveQueryParameters()
+    {
+        Route::get('/', function () {
+            $collection = collect([new Post(['id' => 5, 'title' => 'Test Title']), new Post(['id' => 6, 'title' => 'Hello'])]);
+            $paginator = new CursorPaginator(
+                $collection, 1, null, ['parameters' => ['id']]
+            );
+
+            return PostCollectionResource::make($paginator)->withQuery(['author' => 'Taylor']);
+        });
+
+        $response = $this->withoutExceptionHandling()->get(
+            '/?framework=laravel&author=Otwell', ['Accept' => 'application/json']
+        );
+
+        $response->assertStatus(200);
+
+        $response->assertJson([
+            'data' => [
+                [
+                    'id' => 5,
+                    'title' => 'Test Title',
+                ],
+            ],
+            'links' => [
+                'first' => null,
+                'last' => null,
+                'prev' => null,
+                'next' => '/?author=Taylor&cursor='.(new Cursor(['id' => 5]))->encode(),
+            ],
+            'meta' => [
+                'path' => '/',
+                'per_page' => 1,
+            ],
+        ]);
+    }
+
     public function testToJsonMayBeLeftOffOfCollection()
     {
         Route::get('/', function () {
@@ -705,6 +968,68 @@ class ResourceTest extends TestCase
         });
     }
 
+    public function testCollectionResourceWithPaginationInfomation()
+    {
+        $posts = collect([
+            new Post(['id' => 5, 'title' => 'Test Title']),
+        ]);
+
+        Route::get('/', function () use ($posts) {
+            return new PostCollectionResourceWithPaginationInformation(new LengthAwarePaginator($posts, 10, 1, 1));
+        });
+
+        $response = $this->withoutExceptionHandling()->get(
+            '/',
+            ['Accept' => 'application/json']
+        );
+
+        $response->assertStatus(200);
+
+        $response->assertJson([
+            'data' => [
+                [
+                    'id' => 5,
+                    'title' => 'Test Title',
+                ],
+            ],
+            'current_page' => 1,
+            'per_page' => 1,
+            'total_page' => 10,
+            'total' => 10,
+        ]);
+    }
+
+    public function testResourceWithPaginationInfomation()
+    {
+        $posts = collect([
+            new Post(['id' => 5, 'title' => 'Test Title']),
+        ]);
+
+        Route::get('/', function () use ($posts) {
+            return PostResourceWithAnonymousResourceCollectionWithPaginationInformation::collection(new LengthAwarePaginator($posts, 10, 1, 1));
+        });
+
+        $response = $this->withoutExceptionHandling()->get(
+            '/',
+            ['Accept' => 'application/json']
+        );
+
+        $response->assertStatus(200);
+
+        $response->assertJson([
+            'data' => [
+                [
+                    'id' => 5,
+                    'title' => 'Test Title',
+                ],
+            ],
+            'current_page' => 1,
+            'per_page' => 1,
+            'total_page' => 10,
+            'total' => 10,
+        ]);
+    }
+
     public function testCollectionResourcesAreCountable()
     {
         $posts = collect([
@@ -715,7 +1040,7 @@ class ResourceTest extends TestCase
         $collection = new PostCollectionResource($posts);
 
         $this->assertCount(2, $collection);
-        $this->assertSame(2, count($collection));
+        $this->assertCount(2, $collection);
     }
 
     public function testKeysArePreservedIfTheResourceIsFlaggedToPreserveKeys()
@@ -791,7 +1116,8 @@ class ResourceTest extends TestCase
 
     public function testLeadingMergeKeyedValueIsMergedCorrectly()
     {
-        $filter = new class {
+        $filter = new class
+        {
             use ConditionallyLoadsAttributes;
 
             public function work()
@@ -809,9 +1135,20 @@ class ResourceTest extends TestCase
         ], $results);
     }
 
+    public function testPostTooLargeException()
+    {
+        $this->expectException(PostTooLargeException::class);
+
+        $request = Mockery::mock(Request::class, ['server' => ['CONTENT_LENGTH' => '2147483640']]);
+        $post = new ValidatePostSize;
+        $post->handle($request, function () {
+        });
+    }
+
     public function testLeadingMergeKeyedValueIsMergedCorrectlyWhenFirstValueIsMissing()
     {
-        $filter = new class {
+        $filter = new class
+        {
             use ConditionallyLoadsAttributes;
 
             public function work()
@@ -835,7 +1172,8 @@ class ResourceTest extends TestCase
 
     public function testLeadingMergeValueIsMergedCorrectly()
     {
-        $filter = new class {
+        $filter = new class
+        {
             use ConditionallyLoadsAttributes;
 
             public function work()
@@ -860,7 +1198,8 @@ class ResourceTest extends TestCase
 
     public function testMergeValuesMayBeMissing()
     {
-        $filter = new class {
+        $filter = new class
+        {
             use ConditionallyLoadsAttributes;
 
             public function work()
@@ -885,7 +1224,8 @@ class ResourceTest extends TestCase
 
     public function testInitialMergeValuesMayBeMissing()
     {
-        $filter = new class {
+        $filter = new class
+        {
             use ConditionallyLoadsAttributes;
 
             public function work()
@@ -910,7 +1250,8 @@ class ResourceTest extends TestCase
 
     public function testMergeValueCanMergeJsonSerializable()
     {
-        $filter = new class {
+        $filter = new class
+        {
             use ConditionallyLoadsAttributes;
 
             public function work()
@@ -941,7 +1282,8 @@ class ResourceTest extends TestCase
 
     public function testMergeValueCanMergeCollectionOfJsonSerializable()
     {
-        $filter = new class {
+        $filter = new class
+        {
             use ConditionallyLoadsAttributes;
 
             public function work()
@@ -967,7 +1309,8 @@ class ResourceTest extends TestCase
 
     public function testAllMergeValuesMayBeMissing()
     {
-        $filter = new class {
+        $filter = new class
+        {
             use ConditionallyLoadsAttributes;
 
             public function work()
@@ -992,7 +1335,8 @@ class ResourceTest extends TestCase
 
     public function testNestedMerges()
     {
-        $filter = new class {
+        $filter = new class
+        {
             use ConditionallyLoadsAttributes;
 
             public function work()
