@@ -3,11 +3,14 @@
 namespace Illuminate\Tests\Queue;
 
 use Illuminate\Container\Container;
+use Illuminate\Contracts\Events\Dispatcher;
 use Illuminate\Foundation\Testing\Concerns\InteractsWithRedis;
+use Illuminate\Queue\Events\JobQueued;
 use Illuminate\Queue\Jobs\RedisJob;
 use Illuminate\Queue\RedisQueue;
 use Illuminate\Support\Carbon;
 use Illuminate\Support\InteractsWithTime;
+use Illuminate\Support\Str;
 use Mockery as m;
 use PHPUnit\Framework\TestCase;
 
@@ -16,10 +19,15 @@ class RedisQueueIntegrationTest extends TestCase
     use InteractsWithRedis, InteractsWithTime;
 
     /**
-     * @var RedisQueue
+     * @var \Illuminate\Queue\RedisQueue
      */
     private $queue;
 
+    /**
+     * @var \Mockery\MockInterface|\Mockery\LegacyMockInterface
+     */
+    private $container;
+
     protected function setUp(): void
     {
         Carbon::setTestNow(Carbon::now());
@@ -38,7 +46,7 @@ class RedisQueueIntegrationTest extends TestCase
     /**
      * @dataProvider redisDriverProvider
      *
-     * @param string $driver
+     * @param  string  $driver
      */
     public function testExpiredJobsArePopped($driver)
     {
@@ -56,6 +64,8 @@ class RedisQueueIntegrationTest extends TestCase
         $this->queue->later(-300, $jobs[2]);
         $this->queue->later(-100, $jobs[3]);
 
+        $this->container->shouldHaveReceived('bound')->with('events')->times(4);
+
         $this->assertEquals($jobs[2], unserialize(json_decode($this->queue->pop()->getRawBody())->data->command));
         $this->assertEquals($jobs[1], unserialize(json_decode($this->queue->pop()->getRawBody())->data->command));
         $this->assertEquals($jobs[3], unserialize(json_decode($this->queue->pop()->getRawBody())->data->command));
@@ -75,10 +85,6 @@ class RedisQueueIntegrationTest extends TestCase
      */
     public function testBlockingPop($driver)
     {
-        if (! function_exists('pcntl_fork')) {
-            $this->markTestSkipped('Skipping since the pcntl extension is not available');
-        }
-
         $this->tearDownRedis();
 
         if ($pid = pcntl_fork() > 0) {
@@ -116,7 +122,7 @@ class RedisQueueIntegrationTest extends TestCase
     /**
      * @dataProvider redisDriverProvider
      *
-     * @param string $driver
+     * @param  string  $driver
      */
     public function testPopProperlyPopsJobOffOfRedis($driver)
     {
@@ -128,7 +134,7 @@ class RedisQueueIntegrationTest extends TestCase
 
         // Pop and check it is popped correctly
         $before = $this->currentTime();
-        /** @var RedisJob $redisJob */
+        /** @var \Illuminate\Queue\Jobs\RedisJob $redisJob */
         $redisJob = $this->queue->pop();
         $after = $this->currentTime();
 
@@ -151,7 +157,7 @@ class RedisQueueIntegrationTest extends TestCase
     /**
      * @dataProvider redisDriverProvider
      *
-     * @param string $driver
+     * @param  string  $driver
      */
     public function testPopProperlyPopsDelayedJobOffOfRedis($driver)
     {
@@ -178,17 +184,18 @@ class RedisQueueIntegrationTest extends TestCase
     /**
      * @dataProvider redisDriverProvider
      *
-     * @param string $driver
+     * @param  string  $driver
      */
     public function testPopPopsDelayedJobOffOfRedisWhenExpireNull($driver)
     {
-        $this->queue = new RedisQueue($this->redis[$driver], 'default', null, null);
-        $this->queue->setContainer(m::mock(Container::class));
+        $this->setQueue($driver, 'default', null, null);
 
         // Push an item into queue
         $job = new RedisQueueIntegrationTestJob(10);
         $this->queue->later(-10, $job);
 
+        $this->container->shouldHaveReceived('bound')->with('events')->once();
+
         // Pop and check it is popped correctly
         $before = $this->currentTime();
         $this->assertEquals($job, unserialize(json_decode($this->queue->pop()->getRawBody())->data->command));
@@ -207,7 +214,7 @@ class RedisQueueIntegrationTest extends TestCase
     /**
      * @dataProvider redisDriverProvider
      *
-     * @param string $driver
+     * @param  string  $driver
      */
     public function testBlockingPopProperlyPopsJobOffOfRedis($driver)
     {
@@ -218,7 +225,7 @@ class RedisQueueIntegrationTest extends TestCase
         $this->queue->push($job);
 
         // Pop and check it is popped correctly
-        /** @var RedisJob $redisJob */
+        /** @var \Illuminate\Queue\Jobs\RedisJob $redisJob */
         $redisJob = $this->queue->pop();
 
         $this->assertNotNull($redisJob);
@@ -228,10 +235,14 @@ class RedisQueueIntegrationTest extends TestCase
     /**
      * @dataProvider redisDriverProvider
      *
-     * @param string $driver
+     * @param  string  $driver
      */
     public function testBlockingPopProperlyPopsExpiredJobs($driver)
     {
+        Str::createUuidsUsing(function () {
+            return 'uuid';
+        });
+
         $this->setQueue($driver, 'default', null, 60, 5);
 
         $jobs = [
@@ -248,21 +259,24 @@ class RedisQueueIntegrationTest extends TestCase
         $this->assertEquals(0, $this->redis[$driver]->connection()->llen('queues:default:notify'));
         $this->assertEquals(0, $this->redis[$driver]->connection()->zcard('queues:default:delayed'));
         $this->assertEquals(2, $this->redis[$driver]->connection()->zcard('queues:default:reserved'));
+
+        Str::createUuidsNormally();
     }
 
     /**
      * @dataProvider redisDriverProvider
      *
-     * @param string $driver
+     * @param  string  $driver
      */
     public function testNotExpireJobsWhenExpireNull($driver)
     {
-        $this->queue = new RedisQueue($this->redis[$driver], 'default', null, null);
-        $this->queue->setContainer(m::mock(Container::class));
+        $this->setQueue($driver, 'default', null, null);
 
         // Make an expired reserved job
         $failed = new RedisQueueIntegrationTestJob(-20);
         $this->queue->push($failed);
+        $this->container->shouldHaveReceived('bound')->with('events')->once();
+
         $beforeFailPop = $this->currentTime();
         $this->queue->pop();
         $afterFailPop = $this->currentTime();
@@ -270,6 +284,7 @@ class RedisQueueIntegrationTest extends TestCase
         // Push an item into queue
         $job = new RedisQueueIntegrationTestJob(10);
         $this->queue->push($job);
+        $this->container->shouldHaveReceived('bound')->with('events')->times(2);
 
         // Pop and check it is popped correctly
         $before = $this->currentTime();
@@ -298,16 +313,16 @@ class RedisQueueIntegrationTest extends TestCase
     /**
      * @dataProvider redisDriverProvider
      *
-     * @param string $driver
+     * @param  string  $driver
      */
     public function testExpireJobsWhenExpireSet($driver)
     {
-        $this->queue = new RedisQueue($this->redis[$driver], 'default', null, 30);
-        $this->queue->setContainer(m::mock(Container::class));
+        $this->setQueue($driver, 'default', null, 30);
 
         // Push an item into queue
         $job = new RedisQueueIntegrationTestJob(10);
         $this->queue->push($job);
+        $this->container->shouldHaveReceived('bound')->with('events')->once();
 
         // Pop and check it is popped correctly
         $before = $this->currentTime();
@@ -327,24 +342,24 @@ class RedisQueueIntegrationTest extends TestCase
     /**
      * @dataProvider redisDriverProvider
      *
-     * @param string $driver
+     * @param  string  $driver
      */
     public function testRelease($driver)
     {
         $this->setQueue($driver);
 
-        //push a job into queue
+        // push a job into queue
         $job = new RedisQueueIntegrationTestJob(30);
         $this->queue->push($job);
 
-        //pop and release the job
+        // pop and release the job
         /** @var \Illuminate\Queue\Jobs\RedisJob $redisJob */
         $redisJob = $this->queue->pop();
         $before = $this->currentTime();
         $redisJob->release(1000);
         $after = $this->currentTime();
 
-        //check the content of delayed queue
+        // check the content of delayed queue
         $this->assertEquals(1, $this->redis[$driver]->connection()->zcard('queues:default:delayed'));
 
         $results = $this->redis[$driver]->connection()->zrangebyscore('queues:default:delayed', -INF, INF, ['withscores' => true]);
@@ -361,14 +376,14 @@ class RedisQueueIntegrationTest extends TestCase
         $this->assertEquals(1, $decoded->attempts);
         $this->assertEquals($job, unserialize($decoded->data->command));
 
-        //check if the queue has no ready item yet
+        // check if the queue has no ready item yet
         $this->assertNull($this->queue->pop());
     }
 
     /**
      * @dataProvider redisDriverProvider
      *
-     * @param string $driver
+     * @param  string  $driver
      */
     public function testReleaseInThePast($driver)
     {
@@ -376,7 +391,7 @@ class RedisQueueIntegrationTest extends TestCase
         $job = new RedisQueueIntegrationTestJob(30);
         $this->queue->push($job);
 
-        /** @var RedisJob $redisJob */
+        /** @var \Illuminate\Queue\Jobs\RedisJob $redisJob */
         $redisJob = $this->queue->pop();
         $redisJob->release(-3);
 
@@ -386,7 +401,7 @@ class RedisQueueIntegrationTest extends TestCase
     /**
      * @dataProvider redisDriverProvider
      *
-     * @param string $driver
+     * @param  string  $driver
      */
     public function testDelete($driver)
     {
@@ -410,7 +425,27 @@ class RedisQueueIntegrationTest extends TestCase
     /**
      * @dataProvider redisDriverProvider
      *
-     * @param string $driver
+     * @param  string  $driver
+     */
+    public function testClear($driver)
+    {
+        $this->setQueue($driver);
+
+        $job1 = new RedisQueueIntegrationTestJob(30);
+        $job2 = new RedisQueueIntegrationTestJob(40);
+
+        $this->queue->push($job1);
+        $this->queue->push($job2);
+
+        $this->assertEquals(2, $this->queue->clear(null));
+        $this->assertEquals(0, $this->queue->size());
+        $this->assertEquals(0, $this->redis[$driver]->connection()->llen('queues:default:notify'));
+    }
+
+    /**
+     * @dataProvider redisDriverProvider
+     *
+     * @param  string  $driver
      */
     public function testSize($driver)
     {
@@ -429,16 +464,66 @@ class RedisQueueIntegrationTest extends TestCase
     }
 
     /**
-     * @param string $driver
+     * @dataProvider redisDriverProvider
+     *
+     * @param  string  $driver
+     */
+    public function testPushJobQueuedEvent($driver)
+    {
+        $events = m::mock(Dispatcher::class);
+        $events->shouldReceive('dispatch')->withArgs(function (JobQueued $jobQueued) {
+            $this->assertInstanceOf(RedisQueueIntegrationTestJob::class, $jobQueued->job);
+            $this->assertIsString(RedisQueueIntegrationTestJob::class, $jobQueued->id);
+
+            return true;
+        })->andReturnNull()->once();
+
+        $container = m::mock(Container::class);
+        $container->shouldReceive('bound')->with('events')->andReturn(true)->once();
+        $container->shouldReceive('offsetGet')->with('events')->andReturn($events)->once();
+
+        $queue = new RedisQueue($this->redis[$driver]);
+        $queue->setContainer($container);
+
+        $queue->push(new RedisQueueIntegrationTestJob(5));
+    }
+
+    /**
+     * @dataProvider redisDriverProvider
+     *
+     * @param  string  $driver
+     */
+    public function testBulkJobQueuedEvent($driver)
+    {
+        $events = m::mock(Dispatcher::class);
+        $events->shouldReceive('dispatch')->with(m::type(JobQueued::class))->andReturnNull()->times(3);
+
+        $container = m::mock(Container::class);
+        $container->shouldReceive('bound')->with('events')->andReturn(true)->times(3);
+        $container->shouldReceive('offsetGet')->with('events')->andReturn($events)->times(3);
+
+        $queue = new RedisQueue($this->redis[$driver]);
+        $queue->setContainer($container);
+
+        $queue->bulk([
+            new RedisQueueIntegrationTestJob(5),
+            new RedisQueueIntegrationTestJob(10),
+            new RedisQueueIntegrationTestJob(15),
+        ]);
+    }
+
+    /**
+     * @param  string  $driver
      * @param  string  $default
-     * @param  string  $connection
+     * @param  string|null  $connection
      * @param  int  $retryAfter
      * @param  int|null  $blockFor
      */
     private function setQueue($driver, $default = 'default', $connection = null, $retryAfter = 60, $blockFor = null)
     {
         $this->queue = new RedisQueue($this->redis[$driver], $default, $connection, $retryAfter, $blockFor);
-        $this->queue->setContainer(m::mock(Container::class));
+        $this->container = m::spy(Container::class);
+        $this->queue->setContainer($this->container);
     }
 }
 
