@@ -3,12 +3,15 @@
 namespace Illuminate\Tests\Routing;
 
 use BadMethodCallException;
+use FooController;
 use Illuminate\Container\Container;
 use Illuminate\Contracts\Events\Dispatcher;
 use Illuminate\Http\Request;
+use Illuminate\Routing\Route;
 use Illuminate\Routing\Router;
 use Mockery as m;
 use PHPUnit\Framework\TestCase;
+use Stringable;
 
 class RouteRegistrarTest extends TestCase
 {
@@ -60,6 +63,80 @@ class RouteRegistrarTest extends TestCase
         $this->assertEquals(['seven'], $this->getRoute()->middleware());
     }
 
+    public function testNullNamespaceIsRespected()
+    {
+        $this->router->middleware(['one'])->namespace(null)->get('users', function () {
+            return 'all-users';
+        });
+
+        $this->assertNull($this->getRoute()->getAction()['namespace']);
+    }
+
+    public function testMiddlewareAsStringableObject()
+    {
+        $one = new class implements Stringable
+        {
+            public function __toString()
+            {
+                return 'one';
+            }
+        };
+
+        $this->router->middleware($one)->get('users', function () {
+            return 'all-users';
+        });
+
+        $this->seeResponse('all-users', Request::create('users', 'GET'));
+        $this->assertSame(['one'], $this->getRoute()->middleware());
+    }
+
+    public function testMiddlewareAsStringableObjectOnRouteInstance()
+    {
+        $one = new class implements Stringable
+        {
+            public function __toString()
+            {
+                return 'one';
+            }
+        };
+
+        $this->router->get('users', function () {
+            return 'all-users';
+        })->middleware($one);
+
+        $this->seeResponse('all-users', Request::create('users', 'GET'));
+        $this->assertSame(['one'], $this->getRoute()->middleware());
+    }
+
+    public function testMiddlewareAsArrayWithStringables()
+    {
+        $one = new class implements Stringable
+        {
+            public function __toString()
+            {
+                return 'one';
+            }
+        };
+
+        $this->router->middleware([$one, 'two'])->get('users', function () {
+            return 'all-users';
+        });
+
+        $this->seeResponse('all-users', Request::create('users', 'GET'));
+        $this->assertSame(['one', 'two'], $this->getRoute()->middleware());
+    }
+
+    public function testWithoutMiddlewareRegistration()
+    {
+        $this->router->middleware(['one', 'two'])->get('users', function () {
+            return 'all-users';
+        })->withoutMiddleware('one');
+
+        $this->seeResponse('all-users', Request::create('users', 'GET'));
+
+        $this->assertEquals(['one'], $this->getRoute()->excludedMiddleware());
+    }
+
     public function testCanRegisterGetRouteWithClosureAction()
     {
         $this->router->middleware('get-middleware')->get('users', function () {
@@ -138,6 +215,25 @@ class RouteRegistrarTest extends TestCase
         $this->seeMiddleware('controller-middleware');
     }
 
+    public function testCanRegisterNamespacedGroupRouteWithControllerActionArray()
+    {
+        $this->router->group(['namespace' => 'WhatEver'], function () {
+            $this->router->middleware('controller-middleware')
+                ->get('users', [RouteRegistrarControllerStub::class, 'index']);
+        });
+
+        $this->seeResponse('controller', Request::create('users', 'GET'));
+        $this->seeMiddleware('controller-middleware');
+
+        $this->router->group(['namespace' => 'WhatEver'], function () {
+            $this->router->middleware('controller-middleware')
+                ->get('users', ['\\'.RouteRegistrarControllerStub::class, 'index']);
+        });
+
+        $this->seeResponse('controller', Request::create('users', 'GET'));
+        $this->seeMiddleware('controller-middleware');
+    }
+
     public function testCanRegisterRouteWithArrayAndControllerAction()
     {
         $this->router->middleware('controller-middleware')->put('users', [
@@ -160,6 +256,38 @@ class RouteRegistrarTest extends TestCase
         $this->seeMiddleware('group-middleware');
     }
 
+    public function testCanRegisterGroupWithoutMiddleware()
+    {
+        $this->router->withoutMiddleware('one')->group(function ($router) {
+            $router->get('users', function () {
+                return 'all-users';
+            })->middleware(['one', 'two']);
+        });
+
+        $this->seeResponse('all-users', Request::create('users', 'GET'));
+        $this->assertEquals(['one'], $this->getRoute()->excludedMiddleware());
+    }
+
+    public function testCanRegisterGroupWithStringableMiddleware()
+    {
+        $one = new class implements Stringable
+        {
+            public function __toString()
+            {
+                return 'one';
+            }
+        };
+
+        $this->router->middleware($one)->group(function ($router) {
+            $router->get('users', function () {
+                return 'all-users';
+            });
+        });
+
+        $this->seeResponse('all-users', Request::create('users', 'GET'));
+        $this->seeMiddleware('one');
+    }
+
     public function testCanRegisterGroupWithNamespace()
     {
         $this->router->namespace('App\Http\Controllers')->group(function ($router) {
@@ -220,6 +348,89 @@ class RouteRegistrarTest extends TestCase
         $this->assertSame('api.users', $this->getRoute()->getName());
     }
 
+    public function testCanRegisterGroupWithController()
+    {
+        $this->router->controller(RouteRegistrarControllerStub::class)->group(function ($router) {
+            $router->get('users', 'index');
+        });
+
+        $this->assertSame(
+            RouteRegistrarControllerStub::class.'@index',
+            $this->getRoute()->getAction()['uses']
+        );
+    }
+
+    public function testCanOverrideGroupControllerWithStringSyntax()
+    {
+        $this->router->controller(RouteRegistrarControllerStub::class)->group(function ($router) {
+            $router->get('users', 'UserController@index');
+        });
+
+        $this->assertSame(
+            'UserController@index',
+            $this->getRoute()->getAction()['uses']
+        );
+    }
+
+    public function testCanOverrideGroupControllerWithClosureSyntax()
+    {
+        $this->router->controller(RouteRegistrarControllerStub::class)->group(function ($router) {
+            $router->get('users', function () {
+                return 'hello world';
+            });
+        });
+
+        $this->seeResponse('hello world', Request::create('users', 'GET'));
+    }
+
+    public function testCanOverrideGroupControllerWithInvokableControllerSyntax()
+    {
+        $this->router->controller(RouteRegistrarControllerStub::class)->group(function ($router) {
+            $router->get('users', InvokableRouteRegistrarControllerStub::class);
+        });
+
+        $this->assertSame(
+            InvokableRouteRegistrarControllerStub::class.'@__invoke',
+            $this->getRoute()->getAction()['uses']
+        );
+    }
+
+    public function testWillUseTheLatestGroupController()
+    {
+        $this->router->controller(RouteRegistrarControllerStub::class)->group(function ($router) {
+            $router->group(['controller' => FooController::class], function ($router) {
+                $router->get('users', 'index');
+            });
+        });
+
+        $this->assertSame(
+            FooController::class.'@index',
+            $this->getRoute()->getAction()['uses']
+        );
+    }
+
+    public function testCanOverrideGroupControllerWithArraySyntax()
+    {
+        $this->router->controller(RouteRegistrarControllerStub::class)->group(function ($router) {
+            $router->get('users', [FooController::class, 'index']);
+        });
+
+        $this->assertSame(
+            FooController::class.'@index',
+            $this->getRoute()->getAction()['uses']
+        );
+    }
+
+    public function testRouteGroupingWithoutPrefix()
+    {
+        $this->router->group([], function ($router) {
+            $router->prefix('bar')->get('baz', ['as' => 'baz', function () {
+                return 'hello';
+            }]);
+        });
+        $this->seeResponse('hello', Request::create('bar/baz', 'GET'));
+    }
+
     public function testRegisteringNonApprovedAttributesThrows()
     {
         $this->expectException(BadMethodCallException::class);
@@ -242,9 +453,9 @@ class RouteRegistrarTest extends TestCase
     public function testCanRegisterResourcesWithExceptOption()
     {
         $this->router->resources([
-            'resource-one'      => RouteRegistrarControllerStubOne::class,
-            'resource-two'      => RouteRegistrarControllerStubTwo::class,
-            'resource-three'    => RouteRegistrarControllerStubThree::class,
+            'resource-one' => RouteRegistrarControllerStubOne::class,
+            'resource-two' => RouteRegistrarControllerStubTwo::class,
+            'resource-three' => RouteRegistrarControllerStubThree::class,
         ], ['except' => ['create', 'show']]);
 
         $this->assertCount(15, $this->router->getRoutes());
@@ -264,9 +475,9 @@ class RouteRegistrarTest extends TestCase
     public function testCanRegisterResourcesWithOnlyOption()
     {
         $this->router->resources([
-            'resource-one'      => RouteRegistrarControllerStubOne::class,
-            'resource-two'      => RouteRegistrarControllerStubTwo::class,
-            'resource-three'    => RouteRegistrarControllerStubThree::class,
+            'resource-one' => RouteRegistrarControllerStubOne::class,
+            'resource-two' => RouteRegistrarControllerStubTwo::class,
+            'resource-three' => RouteRegistrarControllerStubThree::class,
         ], ['only' => ['create', 'show']]);
 
         $this->assertCount(6, $this->router->getRoutes());
@@ -286,9 +497,9 @@ class RouteRegistrarTest extends TestCase
     public function testCanRegisterResourcesWithoutOption()
     {
         $this->router->resources([
-            'resource-one'      => RouteRegistrarControllerStubOne::class,
-            'resource-two'      => RouteRegistrarControllerStubTwo::class,
-            'resource-three'    => RouteRegistrarControllerStubThree::class,
+            'resource-one' => RouteRegistrarControllerStubOne::class,
+            'resource-two' => RouteRegistrarControllerStubTwo::class,
+            'resource-three' => RouteRegistrarControllerStubThree::class,
         ]);
 
         $this->assertCount(21, $this->router->getRoutes());
@@ -304,6 +515,24 @@ class RouteRegistrarTest extends TestCase
         }
     }
 
+    public function testCanRegisterResourceWithMissingOption()
+    {
+        $this->router->middleware('resource-middleware')
+            ->resource('users', RouteRegistrarControllerStub::class)
+            ->missing(function () {
+                return 'missing';
+            });
+
+        $this->assertIsCallable($this->router->getRoutes()->getByName('users.show')->getMissing());
+        $this->assertIsCallable($this->router->getRoutes()->getByName('users.edit')->getMissing());
+        $this->assertIsCallable($this->router->getRoutes()->getByName('users.update')->getMissing());
+        $this->assertIsCallable($this->router->getRoutes()->getByName('users.destroy')->getMissing());
+
+        $this->assertNull($this->router->getRoutes()->getByName('users.index')->getMissing());
+        $this->assertNull($this->router->getRoutes()->getByName('users.create')->getMissing());
+        $this->assertNull($this->router->getRoutes()->getByName('users.store')->getMissing());
+    }
+
     public function testCanAccessRegisteredResourceRoutesAsRouteCollection()
     {
         $resource = $this->router->middleware('resource-middleware')
@@ -355,6 +584,31 @@ class RouteRegistrarTest extends TestCase
         $this->assertFalse($this->router->getRoutes()->hasNamedRoute('users.tasks.show'));
     }
 
+    public function testCanSetScopedOptionOnRegisteredResource()
+    {
+        $this->router->resource('users.tasks', RouteRegistrarControllerStub::class)->scoped();
+        $this->assertSame(
+            ['user' => null],
+            $this->router->getRoutes()->getByName('users.tasks.index')->bindingFields()
+        );
+        $this->assertSame(
+            ['user' => null, 'task' => null],
+            $this->router->getRoutes()->getByName('users.tasks.show')->bindingFields()
+        );
+
+        $this->router->resource('users.tasks', RouteRegistrarControllerStub::class)->scoped([
+            'task' => 'slug',
+        ]);
+        $this->assertSame(
+            ['user' => null],
+            $this->router->getRoutes()->getByName('users.tasks.index')->bindingFields()
+        );
+        $this->assertSame(
+            ['user' => null, 'task' => 'slug'],
+            $this->router->getRoutes()->getByName('users.tasks.show')->bindingFields()
+        );
+    }
+
     public function testCanExcludeMethodsOnRegisteredApiResource()
     {
         $this->router->apiResource('users', RouteRegistrarControllerStub::class)
@@ -369,9 +623,9 @@ class RouteRegistrarTest extends TestCase
     public function testCanRegisterApiResourcesWithExceptOption()
     {
         $this->router->apiResources([
-            'resource-one'      => RouteRegistrarControllerStubOne::class,
-            'resource-two'      => RouteRegistrarControllerStubTwo::class,
-            'resource-three'    => RouteRegistrarControllerStubThree::class,
+            'resource-one' => RouteRegistrarControllerStubOne::class,
+            'resource-two' => RouteRegistrarControllerStubTwo::class,
+            'resource-three' => RouteRegistrarControllerStubThree::class,
         ], ['except' => ['create', 'show']]);
 
         $this->assertCount(12, $this->router->getRoutes());
@@ -391,9 +645,9 @@ class RouteRegistrarTest extends TestCase
     public function testCanRegisterApiResourcesWithOnlyOption()
     {
         $this->router->apiResources([
-            'resource-one'      => RouteRegistrarControllerStubOne::class,
-            'resource-two'      => RouteRegistrarControllerStubTwo::class,
-            'resource-three'    => RouteRegistrarControllerStubThree::class,
+            'resource-one' => RouteRegistrarControllerStubOne::class,
+            'resource-two' => RouteRegistrarControllerStubTwo::class,
+            'resource-three' => RouteRegistrarControllerStubThree::class,
         ], ['only' => ['index', 'show']]);
 
         $this->assertCount(6, $this->router->getRoutes());
@@ -413,9 +667,9 @@ class RouteRegistrarTest extends TestCase
     public function testCanRegisterApiResourcesWithoutOption()
     {
         $this->router->apiResources([
-            'resource-one'      => RouteRegistrarControllerStubOne::class,
-            'resource-two'      => RouteRegistrarControllerStubTwo::class,
-            'resource-three'    => RouteRegistrarControllerStubThree::class,
+            'resource-one' => RouteRegistrarControllerStubOne::class,
+            'resource-two' => RouteRegistrarControllerStubTwo::class,
+            'resource-three' => RouteRegistrarControllerStubThree::class,
         ]);
 
         $this->assertCount(15, $this->router->getRoutes());
@@ -511,6 +765,93 @@ class RouteRegistrarTest extends TestCase
         $this->seeMiddleware(RouteRegistrarMiddlewareStub::class);
     }
 
+    public function testResourceWithoutMiddlewareRegistration()
+    {
+        $this->router->resource('users', RouteRegistrarControllerStub::class)
+                     ->only('index')
+                     ->middleware(['one', 'two'])
+                     ->withoutMiddleware('one');
+
+        $this->seeResponse('controller', Request::create('users', 'GET'));
+
+        $this->assertEquals(['one'], $this->getRoute()->excludedMiddleware());
+    }
+
+    public function testResourceWithMiddlewareAsStringable()
+    {
+        $one = new class implements Stringable
+        {
+            public function __toString()
+            {
+                return 'one';
+            }
+        };
+
+        $this->router->resource('users', RouteRegistrarControllerStub::class)
+                     ->only('index')
+                     ->middleware([$one, 'two'])
+                     ->withoutMiddleware('one');
+
+        $this->seeResponse('controller', Request::create('users', 'GET'));
+
+        $this->assertEquals(['one', 'two'], $this->getRoute()->middleware());
+        $this->assertEquals(['one'], $this->getRoute()->excludedMiddleware());
+    }
+
+    public function testResourceWheres()
+    {
+        $wheres = [
+            'user' => '\d+',
+            'test' => '[a-z]+',
+        ];
+
+        $this->router->resource('users', RouteRegistrarControllerStub::class)
+                     ->where($wheres);
+
+        /** @var \Illuminate\Routing\Route $route */
+        foreach ($this->router->getRoutes() as $route) {
+            $this->assertEquals($wheres, $route->wheres);
+        }
+    }
+
+    public function testWhereNumberRegistration()
+    {
+        $wheres = ['foo' => '[0-9]+', 'bar' => '[0-9]+'];
+
+        $this->router->get('/{foo}/{bar}')->whereNumber(['foo', 'bar']);
+        $this->router->get('/api/{bar}/{foo}')->whereNumber(['bar', 'foo']);
+
+        /** @var \Illuminate\Routing\Route $route */
+        foreach ($this->router->getRoutes() as $route) {
+            $this->assertEquals($wheres, $route->wheres);
+        }
+    }
+
+    public function testWhereAlphaRegistration()
+    {
+        $wheres = ['foo' => '[a-zA-Z]+', 'bar' => '[a-zA-Z]+'];
+
+        $this->router->get('/{foo}/{bar}')->whereAlpha(['foo', 'bar']);
+        $this->router->get('/api/{bar}/{foo}')->whereAlpha(['bar', 'foo']);
+
+        /** @var \Illuminate\Routing\Route $route */
+        foreach ($this->router->getRoutes() as $route) {
+            $this->assertEquals($wheres, $route->wheres);
+        }
+    }
+
+    public function testWhereAlphaNumericRegistration()
+    {
+        $wheres = ['1a2b3c' => '[a-zA-Z0-9]+'];
+
+        $this->router->get('/{foo}')->whereAlphaNumeric(['1a2b3c']);
+
+        /** @var \Illuminate\Routing\Route $route */
+        foreach ($this->router->getRoutes() as $route) {
+            $this->assertEquals($wheres, $route->wheres);
+        }
+    }
+
     public function testCanSetRouteName()
     {
         $this->router->as('users.index')->get('users', function () {
@@ -582,6 +923,14 @@ class RouteRegistrarControllerStub
     }
 }
 
+class InvokableRouteRegistrarControllerStub
+{
+    public function __invoke()
+    {
+        return 'controller';
+    }
+}
+
 class RouteRegistrarMiddlewareStub
 {
     //
