@@ -2,6 +2,7 @@
 
 namespace Illuminate\Tests\Routing;
 
+use Closure;
 use DateTime;
 use Exception;
 use Illuminate\Auth\Middleware\Authenticate;
@@ -14,6 +15,7 @@ use Illuminate\Database\Eloquent\ModelNotFoundException;
 use Illuminate\Events\Dispatcher;
 use Illuminate\Http\Exceptions\HttpResponseException;
 use Illuminate\Http\JsonResponse;
+use Illuminate\Http\RedirectResponse;
 use Illuminate\Http\Request;
 use Illuminate\Http\Response;
 use Illuminate\Routing\Controller;
@@ -21,8 +23,10 @@ use Illuminate\Routing\Exceptions\UrlGenerationException;
 use Illuminate\Routing\Middleware\SubstituteBindings;
 use Illuminate\Routing\ResourceRegistrar;
 use Illuminate\Routing\Route;
+use Illuminate\Routing\RouteCollection;
 use Illuminate\Routing\RouteGroup;
 use Illuminate\Routing\Router;
+use Illuminate\Routing\UrlGenerator;
 use Illuminate\Support\Str;
 use LogicException;
 use PHPUnit\Framework\TestCase;
@@ -190,6 +194,30 @@ class RoutingRouteTest extends TestCase
         $this->assertSame('caught', $router->dispatch(Request::create('foo/bar', 'GET'))->getContent());
     }
 
+    public function testMiddlewareCanBeSkipped()
+    {
+        $router = $this->getRouter();
+        $router->aliasMiddleware('web', RoutingTestMiddlewareGroupTwo::class);
+
+        $router->get('foo/bar', ['middleware' => 'web', function () {
+            return 'hello';
+        }])->withoutMiddleware(RoutingTestMiddlewareGroupTwo::class);
+
+        $this->assertSame('hello', $router->dispatch(Request::create('foo/bar', 'GET'))->getContent());
+    }
+
+    public function testMiddlewareCanBeSkippedFromResources()
+    {
+        $router = $this->getRouter();
+        $router->aliasMiddleware('web', RoutingTestMiddlewareGroupTwo::class);
+
+        $router->resource('foo', RouteTestControllerMiddlewareGroupStub::class)
+            ->middleware('web')
+            ->withoutMiddleware(RoutingTestMiddlewareGroupTwo::class);
+
+        $this->assertSame('Hello World', $router->dispatch(Request::create('foo', 'GET'))->getContent());
+    }
+
     public function testMiddlewareWorksIfControllerThrowsHttpResponseException()
     {
         // Before calling controller
@@ -372,6 +400,23 @@ class RoutingRouteTest extends TestCase
         $this->assertNull($route->getAction('unknown_property'));
     }
 
+    public function testResolvingBindingParameters()
+    {
+        $router = $this->getRouter();
+
+        $route = $router->get('foo/{bar:slug}', function () {
+            return 'foo';
+        })->name('foo');
+
+        $this->assertSame('slug', $route->bindingFieldFor('bar'));
+
+        $route = $router->get('foo/{bar:slug}/{baz}', function () {
+            return 'foo';
+        })->name('foo');
+
+        $this->assertNull($route->bindingFieldFor('baz'));
+    }
+
     public function testMacro()
     {
         $router = $this->getRouter();
@@ -421,6 +466,29 @@ class RoutingRouteTest extends TestCase
         unset($_SERVER['__test.route_inject']);
     }
 
+    public function testNullValuesCanBeInjectedIntoRoutes()
+    {
+        $container = new Container;
+        $router = new Router(new Dispatcher, $container);
+        $container->singleton(Registrar::class, function () use ($router) {
+            return $router;
+        });
+
+        $container->bind(RoutingTestUserModel::class, function () {
+        });
+
+        $router->get('foo/{team}/{post}', [
+            'middleware' => SubstituteBindings::class,
+            'uses' => function (?RoutingTestUserModel $userFromContainer, RoutingTestTeamModel $team, $postId) {
+                $this->assertNull($userFromContainer);
+                $this->assertInstanceOf(RoutingTestTeamModel::class, $team);
+                $this->assertSame('bar', $team->value);
+                $this->assertSame('baz', $postId);
+            },
+        ]);
+        $router->dispatch(Request::create('foo/bar/baz', 'GET'))->getContent();
+    }
+
     public function testOptionsResponsesAreGeneratedByDefault()
     {
         $router = $this->getRouter();
@@ -771,6 +839,15 @@ class RoutingRouteTest extends TestCase
         $this->assertFalse($route->matches($request));
     }
 
+    public function testRoutePrefixParameterParsing()
+    {
+        $route = new Route('GET', '/foo', ['prefix' => 'profiles/{user:username}/portfolios', 'uses' => function () {
+            //
+        }]);
+
+        $this->assertSame('profiles/{user}/portfolios/foo', $route->uri());
+    }
+
     public function testDotDoesNotMatchEverything()
     {
         $route = new Route('GET', 'images/{id}.{ext}', function () {
@@ -829,12 +906,13 @@ class RoutingRouteTest extends TestCase
             SubstituteBindings::class,
             Placeholder2::class,
             Authenticate::class,
+            ExampleMiddleware::class,
             Placeholder3::class,
         ];
 
         $router = $this->getRouter();
 
-        $router->middlewarePriority = [Authenticate::class, SubstituteBindings::class, Authorize::class];
+        $router->middlewarePriority = [ExampleMiddlewareContract::class, Authenticate::class, SubstituteBindings::class, Authorize::class];
 
         $route = $router->get('foo', ['middleware' => $middleware, 'uses' => function ($name) {
             return $name;
@@ -842,6 +920,7 @@ class RoutingRouteTest extends TestCase
 
         $this->assertEquals([
             Placeholder1::class,
+            ExampleMiddleware::class,
             Authenticate::class,
             SubstituteBindings::class,
             Placeholder2::class,
@@ -1050,14 +1129,30 @@ class RoutingRouteTest extends TestCase
         $router = $this->getRouter();
         $router->group(['prefix' => 'foo', 'as' => 'Foo::'], function () use ($router) {
             $router->group(['prefix' => 'bar'], function () use ($router) {
-                $router->get('baz', ['as' => 'baz', function () {
+                $router->prefix('foz')->get('baz', ['as' => 'baz', function () {
                     return 'hello';
                 }]);
             });
         });
         $routes = $router->getRoutes();
         $route = $routes->getByName('Foo::baz');
-        $this->assertSame('foo/bar/baz', $route->uri());
+        $this->assertSame('foz/foo/bar/baz', $route->uri());
+    }
+
+    public function testNestedRouteGroupingPrefixing()
+    {
+        /*
+         * nested with layer skipped
+         */
+        $router = $this->getRouter();
+        $router->group(['prefix' => 'foo', 'as' => 'Foo::'], function () use ($router) {
+            $router->prefix('bar')->get('baz', ['as' => 'baz', function () {
+                return 'hello';
+            }]);
+        });
+        $routes = $router->getRoutes();
+        $route = $routes->getByName('Foo::baz');
+        $this->assertSame('bar/foo', $route->getAction('prefix'));
     }
 
     public function testRouteMiddlewareMergeWithMiddlewareAttributesAsStrings()
@@ -1113,6 +1208,18 @@ class RoutingRouteTest extends TestCase
         $routes = $routes->getRoutes();
         $routes[0]->prefix('prefix');
         $this->assertSame('prefix', $routes[0]->uri());
+
+        /*
+         * Prefix homepage with empty prefix
+         */
+        $router = $this->getRouter();
+        $router->get('/', function () {
+            return 'hello';
+        });
+        $routes = $router->getRoutes();
+        $routes = $routes->getRoutes();
+        $routes[0]->prefix('/');
+        $this->assertSame('/', $routes[0]->uri());
     }
 
     public function testRoutePreservingOriginalParametersState()
@@ -1504,6 +1611,7 @@ class RoutingRouteTest extends TestCase
     public function testImplicitBindings()
     {
         $router = $this->getRouter();
+
         $router->get('foo/{bar}', [
             'middleware' => SubstituteBindings::class,
             'uses' => function (RoutingTestUserModel $bar) {
@@ -1512,9 +1620,44 @@ class RoutingRouteTest extends TestCase
                 return $bar->value;
             },
         ]);
+
         $this->assertSame('taylor', $router->dispatch(Request::create('foo/taylor', 'GET'))->getContent());
     }
 
+    public function testParentChildImplicitBindings()
+    {
+        $router = $this->getRouter();
+
+        $router->get('foo/{user}/{post:slug}', [
+            'middleware' => SubstituteBindings::class,
+            'uses' => function (RoutingTestUserModel $user, RoutingTestPostModel $post) {
+                $this->assertInstanceOf(RoutingTestUserModel::class, $user);
+                $this->assertInstanceOf(RoutingTestPostModel::class, $post);
+
+                return $user->value.'|'.$post->value;
+            },
+        ]);
+
+        $this->assertSame('1|test-slug', $router->dispatch(Request::create('foo/1/test-slug', 'GET'))->getContent());
+    }
+
+    public function testParentChildImplicitBindingsProperlyCamelCased()
+    {
+        $router = $this->getRouter();
+
+        $router->get('foo/{user}/{test_team:id}', [
+            'middleware' => SubstituteBindings::class,
+            'uses' => function (RoutingTestUserModel $user, RoutingTestTeamModel $testTeam) {
+                $this->assertInstanceOf(RoutingTestUserModel::class, $user);
+                $this->assertInstanceOf(RoutingTestTeamModel::class, $testTeam);
+
+                return $user->value.'|'.$testTeam->value;
+            },
+        ]);
+
+        $this->assertSame('1|4', $router->dispatch(Request::create('foo/1/4', 'GET'))->getContent());
+    }
+
     public function testImplicitBindingsWithOptionalParameterWithExistingKeyInUri()
     {
         $router = $this->getRouter();
@@ -1529,6 +1672,27 @@ class RoutingRouteTest extends TestCase
         $this->assertSame('taylor', $router->dispatch(Request::create('foo/taylor', 'GET'))->getContent());
     }
 
+    public function testImplicitBindingsWithMissingModelHandledByMissing()
+    {
+        $router = $this->getRouter();
+        $router->get('foo/{bar}', [
+            'middleware' => SubstituteBindings::class,
+            'uses' => function (RouteModelBindingNullStub $bar = null) {
+                $this->assertInstanceOf(RouteModelBindingNullStub::class, $bar);
+
+                return $bar->first();
+            },
+        ])->missing(function () {
+            return new RedirectResponse('/', 302);
+        });
+
+        $request = Request::create('foo/taylor', 'GET');
+
+        $response = $router->dispatch($request);
+        $this->assertTrue($response->isRedirect('/'));
+        $this->assertEquals(302, $response->getStatusCode());
+    }
+
     public function testImplicitBindingsWithOptionalParameterWithNoKeyInUri()
     {
         $router = $this->getRouter();
@@ -1599,6 +1763,28 @@ class RoutingRouteTest extends TestCase
         $this->assertNotInstanceOf(JsonResponse::class, $response);
     }
 
+    public function testRouteFlushController()
+    {
+        $container = new Container;
+        $router = $this->getRouter();
+
+        $router->get('count', ActionCountStub::class);
+        $request = Request::create('count', 'GET');
+
+        $response = $router->dispatch($request);
+        $this->assertSame(1, $response->original['invokedCount']);
+        $this->assertSame(1, $response->original['middlewareInvokedCount']);
+
+        $response = $router->dispatch($request);
+        $this->assertSame(2, $response->original['invokedCount']);
+        $this->assertSame(2, $response->original['middlewareInvokedCount']);
+
+        $request->route()->flushController();
+        $response = $router->dispatch($request);
+        $this->assertSame(1, $response->original['invokedCount']);
+        $this->assertSame(1, $response->original['middlewareInvokedCount']);
+    }
+
     public function testJsonResponseIsReturned()
     {
         $router = $this->getRouter();
@@ -1622,6 +1808,10 @@ class RoutingRouteTest extends TestCase
         $container->singleton(Request::class, function () use ($request) {
             return $request;
         });
+        $urlGenerator = new UrlGenerator(new RouteCollection, $request);
+        $container->singleton(UrlGenerator::class, function () use ($urlGenerator) {
+            return $urlGenerator;
+        });
         $router->get('contact_us', function () {
             throw new Exception('Route should not be reachable.');
         });
@@ -1643,6 +1833,10 @@ class RoutingRouteTest extends TestCase
         $container->singleton(Request::class, function () use ($request) {
             return $request;
         });
+        $urlGenerator = new UrlGenerator(new RouteCollection, $request);
+        $container->singleton(UrlGenerator::class, function () use ($urlGenerator) {
+            return $urlGenerator;
+        });
         $router->get('contact_us', function () {
             throw new Exception('Route should not be reachable.');
         });
@@ -1664,6 +1858,10 @@ class RoutingRouteTest extends TestCase
         $container->singleton(Request::class, function () use ($request) {
             return $request;
         });
+        $urlGenerator = new UrlGenerator(new RouteCollection, $request);
+        $container->singleton(UrlGenerator::class, function () use ($urlGenerator) {
+            return $urlGenerator;
+        });
         $router->get('contact_us', function () {
             throw new Exception('Route should not be reachable.');
         });
@@ -1677,7 +1875,7 @@ class RoutingRouteTest extends TestCase
     public function testRouteRedirectExceptionWhenMissingExpectedParameters()
     {
         $this->expectException(UrlGenerationException::class);
-        $this->expectExceptionMessage('Missing required parameters for [Route: laravel_route_redirect_destination] [URI: users/{user}].');
+        $this->expectExceptionMessage('Missing required parameter for [Route: laravel_route_redirect_destination] [URI: users/{user}] [Missing parameter: user].');
 
         $container = new Container;
         $router = new Router(new Dispatcher, $container);
@@ -1688,6 +1886,10 @@ class RoutingRouteTest extends TestCase
         $container->singleton(Request::class, function () use ($request) {
             return $request;
         });
+        $urlGenerator = new UrlGenerator(new RouteCollection, $request);
+        $container->singleton(UrlGenerator::class, function () use ($urlGenerator) {
+            return $urlGenerator;
+        });
         $router->get('users', function () {
             throw new Exception('Route should not be reachable.');
         });
@@ -1707,6 +1909,10 @@ class RoutingRouteTest extends TestCase
         $container->singleton(Request::class, function () use ($request) {
             return $request;
         });
+        $urlGenerator = new UrlGenerator(new RouteCollection, $request);
+        $container->singleton(UrlGenerator::class, function () use ($urlGenerator) {
+            return $urlGenerator;
+        });
         $router->get('contact_us', function () {
             throw new Exception('Route should not be reachable.');
         });
@@ -1728,6 +1934,10 @@ class RoutingRouteTest extends TestCase
         $container->singleton(Request::class, function () use ($request) {
             return $request;
         });
+        $urlGenerator = new UrlGenerator(new RouteCollection, $request);
+        $container->singleton(UrlGenerator::class, function () use ($urlGenerator) {
+            return $urlGenerator;
+        });
         $router->get('contact_us', function () {
             throw new Exception('Route should not be reachable.');
         });
@@ -1738,6 +1948,24 @@ class RoutingRouteTest extends TestCase
         $this->assertEquals(301, $response->getStatusCode());
     }
 
+    public function testRouteCanMiddlewareCanBeAssigned()
+    {
+        $route = new Route(['GET'], '/', []);
+        $route->middleware(['foo'])->can('create', Route::class);
+
+        $this->assertEquals([
+            'foo',
+            'can:create,Illuminate\Routing\Route',
+        ], $route->middleware());
+
+        $route = new Route(['GET'], '/', []);
+        $route->can('create');
+
+        $this->assertEquals([
+            'can:create',
+        ], $route->middleware());
+    }
+
     protected function getRouter()
     {
         $container = new Container;
@@ -1994,6 +2222,16 @@ class RoutingTestMiddlewareGroupTwo
 
 class RoutingTestUserModel extends Model
 {
+    public function posts()
+    {
+        return new RoutingTestPostModel;
+    }
+
+    public function testTeams()
+    {
+        return new RoutingTestTeamModel;
+    }
+
     public function getRouteKeyName()
     {
         return 'id';
@@ -2017,6 +2255,26 @@ class RoutingTestUserModel extends Model
     }
 }
 
+class RoutingTestPostModel extends Model
+{
+    public function getRouteKeyName()
+    {
+        return 'id';
+    }
+
+    public function where($key, $value)
+    {
+        $this->value = $value;
+
+        return $this;
+    }
+
+    public function first()
+    {
+        return $this;
+    }
+}
+
 class RoutingTestTeamModel extends Model
 {
     public function getRouteKeyName()
@@ -2067,3 +2325,42 @@ class ActionStub
         return 'hello';
     }
 }
+
+class ActionCountStub extends Controller
+{
+    protected $middlewareInvokedCount = 0;
+
+    protected $invokedCount = 0;
+
+    public function __construct()
+    {
+        $this->middleware(function ($request, $next) {
+            $this->middlewareInvokedCount++;
+
+            return $next($request);
+        });
+    }
+
+    public function __invoke()
+    {
+        $this->invokedCount++;
+
+        return [
+            'invokedCount' => $this->invokedCount,
+            'middlewareInvokedCount' => $this->middlewareInvokedCount,
+        ];
+    }
+}
+
+interface ExampleMiddlewareContract
+{
+    //
+}
+
+class ExampleMiddleware implements ExampleMiddlewareContract
+{
+    public function handle($request, Closure $next)
+    {
+        return $next($request);
+    }
+}
