@@ -3,6 +3,7 @@
 namespace Illuminate\Tests\Routing;
 
 use Illuminate\Contracts\Routing\UrlRoutable;
+use Illuminate\Database\Eloquent\Model;
 use Illuminate\Http\Request;
 use Illuminate\Routing\Exceptions\UrlGenerationException;
 use Illuminate\Routing\Route;
@@ -355,6 +356,23 @@ class RoutingUrlGeneratorTest extends TestCase
         $this->assertSame('/foo/routable', $url->route('routable', [$model], false));
     }
 
+    public function testRoutableInterfaceRoutingWithCustomBindingField()
+    {
+        $url = new UrlGenerator(
+            $routes = new RouteCollection,
+            Request::create('http://www.foo.com/')
+        );
+
+        $route = new Route(['GET'], 'foo/{bar:slug}', ['as' => 'routable']);
+        $routes->add($route);
+
+        $model = new RoutableInterfaceStub;
+        $model->key = 'routable';
+
+        $this->assertSame('/foo/test-slug', $url->route('routable', ['bar' => $model], false));
+        $this->assertSame('/foo/test-slug', $url->route('routable', [$model], false));
+    }
+
     public function testRoutableInterfaceRoutingWithSingleParameter()
     {
         $url = new UrlGenerator(
@@ -533,6 +551,61 @@ class RoutingUrlGeneratorTest extends TestCase
         $this->assertSame('http://www.foo.com:8080/foo?test=123', $url->route('foo', $parameters));
     }
 
+    public function provideParametersAndExpectedMeaningfulExceptionMessages()
+    {
+        return [
+            'Missing parameters "one", "two" and "three"' => [
+                [],
+                'Missing required parameters for [Route: foo] [URI: foo/{one}/{two}/{three}/{four?}] [Missing parameters: one, two, three].',
+            ],
+            'Missing parameters "two" and "three"' => [
+                ['one' => '123'],
+                'Missing required parameters for [Route: foo] [URI: foo/{one}/{two}/{three}/{four?}] [Missing parameters: two, three].',
+            ],
+            'Missing parameters "one" and "three"' => [
+                ['two' => '123'],
+                'Missing required parameters for [Route: foo] [URI: foo/{one}/{two}/{three}/{four?}] [Missing parameters: one, three].',
+            ],
+            'Missing parameters "one" and "two"' => [
+                ['three' => '123'],
+                'Missing required parameters for [Route: foo] [URI: foo/{one}/{two}/{three}/{four?}] [Missing parameters: one, two].',
+            ],
+            'Missing parameter "three"' => [
+                ['one' => '123', 'two' => '123'],
+                'Missing required parameter for [Route: foo] [URI: foo/{one}/{two}/{three}/{four?}] [Missing parameter: three].',
+            ],
+            'Missing parameter "two"' => [
+                ['one' => '123', 'three' => '123'],
+                'Missing required parameter for [Route: foo] [URI: foo/{one}/{two}/{three}/{four?}] [Missing parameter: two].',
+            ],
+            'Missing parameter "one"' => [
+                ['two' => '123', 'three' => '123'],
+                'Missing required parameter for [Route: foo] [URI: foo/{one}/{two}/{three}/{four?}] [Missing parameter: one].',
+            ],
+        ];
+    }
+
+    /**
+     * @dataProvider provideParametersAndExpectedMeaningfulExceptionMessages
+     */
+    public function testUrlGenerationThrowsExceptionForMissingParametersWithMeaningfulMessage($parameters, $expectedMeaningfulExceptionMessage)
+    {
+        $this->expectException(UrlGenerationException::class);
+        $this->expectExceptionMessage($expectedMeaningfulExceptionMessage);
+
+        $url = new UrlGenerator(
+            $routes = new RouteCollection,
+            Request::create('http://www.foo.com:8080/')
+        );
+
+        $route = new Route(['GET'], 'foo/{one}/{two}/{three}/{four?}', ['as' => 'foo', function () {
+            //
+        }]);
+        $routes->add($route);
+
+        $url->route('foo', $parameters);
+    }
+
     public function testForceRootUrl()
     {
         $url = new UrlGenerator(
@@ -618,6 +691,28 @@ class RoutingUrlGeneratorTest extends TestCase
         $this->assertFalse($url->hasValidSignature($request));
     }
 
+    public function testSignedUrlImplicitModelBinding()
+    {
+        $url = new UrlGenerator(
+            $routes = new RouteCollection,
+            $request = Request::create('http://www.foo.com/')
+        );
+        $url->setKeyResolver(function () {
+            return 'secret';
+        });
+
+        $route = new Route(['GET'], 'foo/{user:uuid}', ['as' => 'foo', function () {
+            //
+        }]);
+        $routes->add($route);
+
+        $user = new RoutingUrlGeneratorTestUser(['uuid' => '0231d4ac-e9e3-4452-a89a-4427cfb23c3e']);
+
+        $request = Request::create($url->signedRoute('foo', $user));
+
+        $this->assertTrue($url->hasValidSignature($request));
+    }
+
     public function testSignedRelativeUrl()
     {
         $url = new UrlGenerator(
@@ -664,11 +759,33 @@ class RoutingUrlGeneratorTest extends TestCase
 
         Request::create($url->signedRoute('foo', ['signature' => 'bar']));
     }
+
+    public function testSignedUrlParameterCannotBeNamedExpires()
+    {
+        $url = new UrlGenerator(
+            $routes = new RouteCollection,
+            $request = Request::create('http://www.foo.com/')
+        );
+        $url->setKeyResolver(function () {
+            return 'secret';
+        });
+
+        $route = new Route(['GET'], 'foo/{expires}', ['as' => 'foo', function () {
+            //
+        }]);
+        $routes->add($route);
+
+        $this->expectException(InvalidArgumentException::class);
+        $this->expectExceptionMessage('reserved');
+
+        Request::create($url->signedRoute('foo', ['expires' => 253402300799]));
+    }
 }
 
 class RoutableInterfaceStub implements UrlRoutable
 {
     public $key;
+    public $slug = 'test-slug';
 
     public function getRouteKey()
     {
@@ -680,7 +797,12 @@ class RoutableInterfaceStub implements UrlRoutable
         return 'key';
     }
 
-    public function resolveRouteBinding($routeKey)
+    public function resolveRouteBinding($routeKey, $field = null)
+    {
+        //
+    }
+
+    public function resolveChildRouteBinding($childType, $routeKey, $field = null)
     {
         //
     }
@@ -693,3 +815,8 @@ class InvokableActionStub
         return 'hello';
     }
 }
+
+class RoutingUrlGeneratorTestUser extends Model
+{
+    protected $fillable = ['uuid'];
+}
