@@ -6,6 +6,7 @@ use ArrayAccess;
 use Illuminate\Contracts\Support\Htmlable;
 use Illuminate\Support\Env;
 use Illuminate\Support\Optional;
+use LogicException;
 use Mockery as m;
 use PHPUnit\Framework\TestCase;
 use RuntimeException;
@@ -39,6 +40,9 @@ class SupportHelpersTest extends TestCase
         $this->assertSame('foo', value(function () {
             return 'foo';
         }));
+        $this->assertSame('foo', value(function ($arg) {
+            return $arg;
+        }, 'foo'));
     }
 
     public function testObjectGet()
@@ -362,10 +366,63 @@ class SupportHelpersTest extends TestCase
     }
 
     public function testThrow()
+    {
+        $this->expectException(LogicException::class);
+
+        throw_if(true, new LogicException);
+    }
+
+    public function testThrowDefaultException()
     {
         $this->expectException(RuntimeException::class);
 
-        throw_if(true, new RuntimeException);
+        throw_if(true);
+    }
+
+    public function testThrowExceptionWithMessage()
+    {
+        $this->expectException(RuntimeException::class);
+        $this->expectExceptionMessage('test');
+
+        throw_if(true, 'test');
+    }
+
+    public function testThrowExceptionAsStringWithMessage()
+    {
+        $this->expectException(LogicException::class);
+        $this->expectExceptionMessage('test');
+
+        throw_if(true, LogicException::class, 'test');
+    }
+
+    public function testThrowUnless()
+    {
+        $this->expectException(LogicException::class);
+
+        throw_unless(false, new LogicException);
+    }
+
+    public function testThrowUnlessDefaultException()
+    {
+        $this->expectException(RuntimeException::class);
+
+        throw_unless(false);
+    }
+
+    public function testThrowUnlessExceptionWithMessage()
+    {
+        $this->expectException(RuntimeException::class);
+        $this->expectExceptionMessage('test');
+
+        throw_unless(false, 'test');
+    }
+
+    public function testThrowUnlessExceptionAsStringWithMessage()
+    {
+        $this->expectException(LogicException::class);
+        $this->expectExceptionMessage('test');
+
+        throw_unless(false, LogicException::class, 'test');
     }
 
     public function testThrowReturnIfNotThrown()
@@ -385,7 +442,8 @@ class SupportHelpersTest extends TestCase
     {
         $this->assertNull(optional(null)->something());
 
-        $this->assertEquals(10, optional(new class {
+        $this->assertEquals(10, optional(new class
+        {
             public function something()
             {
                 return 10;
@@ -463,10 +521,12 @@ class SupportHelpersTest extends TestCase
 
         $this->assertNull(optional(null)->present()->something());
 
-        $this->assertSame('$10.00', optional(new class {
+        $this->assertSame('$10.00', optional(new class
+        {
             public function present()
             {
-                return new class {
+                return new class
+                {
                     public function something()
                     {
                         return '$10.00';
@@ -492,7 +552,28 @@ class SupportHelpersTest extends TestCase
         $this->assertEquals(2, $attempts);
 
         // Make sure we waited 100ms for the first attempt
-        $this->assertEqualsWithDelta(0.1, microtime(true) - $startTime, 0.02);
+        $this->assertEqualsWithDelta(0.1, microtime(true) - $startTime, 0.03);
+    }
+
+    public function testRetryWithPassingSleepCallback()
+    {
+        $startTime = microtime(true);
+
+        $attempts = retry(3, function ($attempts) {
+            if ($attempts > 2) {
+                return $attempts;
+            }
+
+            throw new RuntimeException;
+        }, function ($attempt) {
+            return $attempt * 100;
+        });
+
+        // Make sure we made three attempts
+        $this->assertEquals(3, $attempts);
+
+        // Make sure we waited 300ms for the first two attempts
+        $this->assertEqualsWithDelta(0.3, microtime(true) - $startTime, 0.03);
     }
 
     public function testRetryWithPassingWhenCallback()
@@ -513,7 +594,7 @@ class SupportHelpersTest extends TestCase
         $this->assertEquals(2, $attempts);
 
         // Make sure we waited 100ms for the first attempt
-        $this->assertEqualsWithDelta(0.1, microtime(true) - $startTime, 0.02);
+        $this->assertEqualsWithDelta(0.1, microtime(true) - $startTime, 0.03);
     }
 
     public function testRetryWithFailingWhenCallback()
@@ -637,11 +718,11 @@ class SupportHelpersTest extends TestCase
         $this->assertSame('x"null"x', env('foo'));
     }
 
-    public function testGetFromENVFirst()
+    public function testGetFromSERVERFirst()
     {
         $_ENV['foo'] = 'From $_ENV';
         $_SERVER['foo'] = 'From $_SERVER';
-        $this->assertSame('From $_ENV', env('foo'));
+        $this->assertSame('From $_SERVER', env('foo'));
     }
 
     public function providesPregReplaceArrayData()
@@ -663,7 +744,9 @@ class SupportHelpersTest extends TestCase
         ];
     }
 
-    /** @dataProvider providesPregReplaceArrayData */
+    /**
+     * @dataProvider providesPregReplaceArrayData
+     */
     public function testPregReplaceArray($pattern, $replacements, $subject, $expectedOutput)
     {
         $this->assertSame(
@@ -712,22 +795,23 @@ class SupportTestArrayAccess implements ArrayAccess
         $this->attributes = $attributes;
     }
 
-    public function offsetExists($offset)
+    public function offsetExists($offset): bool
     {
         return array_key_exists($offset, $this->attributes);
     }
 
+    #[\ReturnTypeWillChange]
     public function offsetGet($offset)
     {
         return $this->attributes[$offset];
     }
 
-    public function offsetSet($offset, $value)
+    public function offsetSet($offset, $value): void
     {
         $this->attributes[$offset] = $value;
     }
 
-    public function offsetUnset($offset)
+    public function offsetUnset($offset): void
     {
         unset($this->attributes[$offset]);
     }
