@@ -2,12 +2,42 @@
 
 namespace Illuminate\Tests\Support;
 
+use Exception;
+use Illuminate\Support\ItemNotFoundException;
 use Illuminate\Support\LazyCollection;
+use Illuminate\Support\MultipleItemsFoundException;
 use PHPUnit\Framework\TestCase;
 use stdClass;
 
 class SupportLazyCollectionIsLazyTest extends TestCase
 {
+    use Concerns\CountsEnumerations;
+
+    public function testMakeWithClosureIsLazy()
+    {
+        [$closure, $recorder] = $this->makeGeneratorFunctionWithRecorder();
+
+        LazyCollection::make($closure);
+
+        $this->assertEquals([], $recorder->all());
+    }
+
+    public function testMakeWithLazyCollectionIsLazy()
+    {
+        $this->assertDoesNotEnumerate(function ($collection) {
+            LazyCollection::make($collection);
+        });
+    }
+
+    public function testMakeWithGeneratorIsNotLazy()
+    {
+        [$closure, $recorder] = $this->makeGeneratorFunctionWithRecorder(5);
+
+        LazyCollection::make($closure());
+
+        $this->assertEquals([1, 2, 3, 4, 5], $recorder->all());
+    }
+
     public function testEagerEnumeratesOnce()
     {
         $this->assertEnumeratesOnce(function ($collection) {
@@ -29,6 +59,29 @@ class SupportLazyCollectionIsLazyTest extends TestCase
         });
     }
 
+    public function testChunkWhileIsLazy()
+    {
+        $collection = LazyCollection::make(['A', 'A', 'B', 'B', 'C', 'C', 'C']);
+
+        $this->assertDoesNotEnumerateCollection($collection, function ($collection) {
+            $collection->chunkWhile(function ($current, $key, $chunk) {
+                return $current === $chunk->last();
+            });
+        });
+
+        $this->assertEnumeratesCollection($collection, 3, function ($collection) {
+            $collection->chunkWhile(function ($current, $key, $chunk) {
+                return $current === $chunk->last();
+            })->first();
+        });
+
+        $this->assertEnumeratesCollectionOnce($collection, function ($collection) {
+            $collection->chunkWhile(function ($current, $key, $chunk) {
+                return $current === $chunk->last();
+            })->all();
+        });
+    }
+
     public function testCollapseIsLazy()
     {
         $collection = LazyCollection::make([
@@ -416,6 +469,25 @@ class SupportLazyCollectionIsLazyTest extends TestCase
         $this->assertEnumerates(5, function ($collection) {
             $collection->has(4);
         });
+
+        $this->assertEnumeratesOnce(function ($collection) {
+            $collection->has('non-existent');
+        });
+    }
+
+    public function testHasAnyIsLazy()
+    {
+        $this->assertEnumerates(5, function ($collection) {
+            $collection->hasAny(4);
+        });
+
+        $this->assertEnumerates(2, function ($collection) {
+            $collection->hasAny([1, 4]);
+        });
+
+        $this->assertEnumeratesOnce(function ($collection) {
+            $collection->hasAny(['non', 'existent']);
+        });
     }
 
     public function testImplodeEnumeratesOnce()
@@ -461,6 +533,13 @@ class SupportLazyCollectionIsLazyTest extends TestCase
         });
     }
 
+    public function testContainsOneItemIsLazy()
+    {
+        $this->assertEnumerates(2, function ($collection) {
+            $collection->containsOneItem();
+        });
+    }
+
     public function testJoinIsLazy()
     {
         $this->assertEnumeratesOnce(function ($collection) {
@@ -739,8 +818,16 @@ class SupportLazyCollectionIsLazyTest extends TestCase
         });
     }
 
-    public function testReduceEnumeratesOnce()
+    public function testReduceIsLazy()
     {
+        $this->assertEnumerates(1, function ($collection) {
+            $this->rescue(function () use ($collection) {
+                $collection->reduce(function ($total, $value) {
+                    throw new Exception('Short-circuit');
+                }, 0);
+            });
+        });
+
         $this->assertEnumeratesOnce(function ($collection) {
             $collection->reduce(function ($total, $value) {
                 return $total + $value;
@@ -748,6 +835,23 @@ class SupportLazyCollectionIsLazyTest extends TestCase
         });
     }
 
+    public function testReduceSpreadIsLazy()
+    {
+        $this->assertEnumerates(1, function ($collection) {
+            $this->rescue(function () use ($collection) {
+                $collection->reduceSpread(function ($one, $two, $value) {
+                    throw new Exception('Short-circuit');
+                }, 0, 0);
+            });
+        });
+
+        $this->assertEnumeratesOnce(function ($collection) {
+            $collection->reduceSpread(function ($total, $max, $value) {
+                return [$total + $value, max($max, $value)];
+            }, 0, 0);
+        });
+    }
+
     public function testRejectIsLazy()
     {
         $this->assertDoesNotEnumerate(function ($collection) {
@@ -839,6 +943,29 @@ class SupportLazyCollectionIsLazyTest extends TestCase
         });
     }
 
+    public function testSlidingIsLazy()
+    {
+        $this->assertDoesNotEnumerate(function ($collection) {
+            $collection->sliding();
+        });
+
+        $this->assertEnumerates(2, function ($collection) {
+            $collection->sliding()->take(1)->all();
+        });
+
+        $this->assertEnumerates(3, function ($collection) {
+            $collection->sliding()->take(2)->all();
+        });
+
+        $this->assertEnumerates(13, function ($collection) {
+            $collection->sliding(3, 5)->take(3)->all();
+        });
+
+        $this->assertEnumeratesOnce(function ($collection) {
+            $collection->sliding()->all();
+        });
+    }
+
     public function testSkipIsLazy()
     {
         $this->assertDoesNotEnumerate(function ($collection) {
@@ -850,6 +977,40 @@ class SupportLazyCollectionIsLazyTest extends TestCase
         });
     }
 
+    public function testSkipUntilIsLazy()
+    {
+        $this->assertDoesNotEnumerate(function ($collection) {
+            $collection->skipUntil(INF);
+        });
+
+        $this->assertEnumerates(10, function ($collection) {
+            $collection->skipUntil(10)->first();
+        });
+
+        $this->assertEnumerates(10, function ($collection) {
+            $collection->skipUntil(function ($item) {
+                return $item === 10;
+            })->first();
+        });
+    }
+
+    public function testSkipWhileIsLazy()
+    {
+        $this->assertDoesNotEnumerate(function ($collection) {
+            $collection->skipWhile(1);
+        });
+
+        $this->assertEnumerates(2, function ($collection) {
+            $collection->skipWhile(1)->first();
+        });
+
+        $this->assertEnumerates(10, function ($collection) {
+            $collection->skipWhile(function ($item) {
+                return $item < 10;
+            })->first();
+        });
+    }
+
     public function testSliceIsLazy()
     {
         $this->assertDoesNotEnumerate(function ($collection) {
@@ -871,6 +1032,35 @@ class SupportLazyCollectionIsLazyTest extends TestCase
         });
     }
 
+    public function testFindFirstOrFailIsLazy()
+    {
+        $this->assertEnumerates(1, function ($collection) {
+            $collection->firstOrFail();
+        });
+
+        $this->assertEnumerates(1, function ($collection) {
+            $collection->firstOrFail(function ($item) {
+                return $item === 1;
+            });
+        });
+
+        $this->assertEnumerates(100, function ($collection) {
+            try {
+                $collection->firstOrFail(function ($item) {
+                    return $item === 101;
+                });
+            } catch (ItemNotFoundException $e) {
+                //
+            }
+        });
+
+        $this->assertEnumerates(2, function ($collection) {
+            $collection->firstOrFail(function ($item) {
+                return $item % 2 === 0;
+            });
+        });
+    }
+
     public function testSomeIsLazy()
     {
         $this->assertEnumerates(5, function ($collection) {
@@ -886,6 +1076,33 @@ class SupportLazyCollectionIsLazyTest extends TestCase
         });
     }
 
+    public function testSoleIsLazy()
+    {
+        $this->assertEnumerates(2, function ($collection) {
+            try {
+                $collection->sole();
+            } catch (MultipleItemsFoundException $e) {
+                //
+            }
+        });
+
+        $this->assertEnumeratesOnce(function ($collection) {
+            $collection->sole(function ($item) {
+                return $item === 1;
+            });
+        });
+
+        $this->assertEnumerates(4, function ($collection) {
+            try {
+                $collection->sole(function ($item) {
+                    return $item % 2 === 0;
+                });
+            } catch (MultipleItemsFoundException $e) {
+                //
+            }
+        });
+    }
+
     public function testSortIsLazy()
     {
         $this->assertDoesNotEnumerate(function ($collection) {
@@ -897,6 +1114,17 @@ class SupportLazyCollectionIsLazyTest extends TestCase
         });
     }
 
+    public function testSortDescIsLazy()
+    {
+        $this->assertDoesNotEnumerate(function ($collection) {
+            $collection->sortDesc();
+        });
+
+        $this->assertEnumeratesOnce(function ($collection) {
+            $collection->sortDesc()->all();
+        });
+    }
+
     public function testSortByIsLazy()
     {
         $this->assertDoesNotEnumerate(function ($collection) {
@@ -978,6 +1206,40 @@ class SupportLazyCollectionIsLazyTest extends TestCase
         });
     }
 
+    public function testTakeUntilIsLazy()
+    {
+        $this->assertDoesNotEnumerate(function ($collection) {
+            $collection->takeUntil(INF);
+        });
+
+        $this->assertEnumerates(10, function ($collection) {
+            $collection->takeUntil(10)->all();
+        });
+
+        $this->assertEnumerates(10, function ($collection) {
+            $collection->takeUntil(function ($item) {
+                return $item === 10;
+            })->all();
+        });
+    }
+
+    public function testTakeWhileIsLazy()
+    {
+        $this->assertDoesNotEnumerate(function ($collection) {
+            $collection->takeWhile(0);
+        });
+
+        $this->assertEnumerates(1, function ($collection) {
+            $collection->takeWhile(0)->all();
+        });
+
+        $this->assertEnumerates(10, function ($collection) {
+            $collection->takeWhile(function ($item) {
+                return $item < 10;
+            })->all();
+        });
+    }
+
     public function testTapDoesNotEnumerate()
     {
         $this->assertDoesNotEnumerate(function ($collection) {
@@ -1245,6 +1507,52 @@ class SupportLazyCollectionIsLazyTest extends TestCase
         });
     }
 
+    public function testWhereNotNullIsLazy()
+    {
+        $data = $this->make([['a' => 1], ['a' => null], ['a' => 2], ['a' => 3]]);
+
+        $this->assertDoesNotEnumerateCollection($data, function ($collection) {
+            $collection->whereNotNull('a');
+        });
+
+        $this->assertEnumeratesCollectionOnce($data, function ($collection) {
+            $collection->whereNotNull('a')->all();
+        });
+
+        $data = $this->make([1, null, 2, null, 3]);
+
+        $this->assertDoesNotEnumerateCollection($data, function ($collection) {
+            $collection->whereNotNull();
+        });
+
+        $this->assertEnumeratesCollectionOnce($data, function ($collection) {
+            $collection->whereNotNull()->all();
+        });
+    }
+
+    public function testWhereNullIsLazy()
+    {
+        $data = $this->make([['a' => 1], ['a' => null], ['a' => 2], ['a' => 3]]);
+
+        $this->assertDoesNotEnumerateCollection($data, function ($collection) {
+            $collection->whereNull('a');
+        });
+
+        $this->assertEnumeratesCollectionOnce($data, function ($collection) {
+            $collection->whereNull('a')->all();
+        });
+
+        $data = $this->make([1, null, 2, null, 3]);
+
+        $this->assertDoesNotEnumerateCollection($data, function ($collection) {
+            $collection->whereNull();
+        });
+
+        $this->assertEnumeratesCollectionOnce($data, function ($collection) {
+            $collection->whereNull()->all();
+        });
+    }
+
     public function testWhereStrictIsLazy()
     {
         $data = $this->make([['a' => 1], ['a' => 2], ['a' => 3], ['a' => 4]]);
@@ -1292,76 +1600,12 @@ class SupportLazyCollectionIsLazyTest extends TestCase
         return new LazyCollection($source);
     }
 
-    protected function assertDoesNotEnumerate(callable $executor)
+    protected function rescue($callback)
     {
-        $this->assertEnumerates(0, $executor);
-    }
-
-    protected function assertDoesNotEnumerateCollection(
-        LazyCollection $collection,
-        callable $executor
-    ) {
-        $this->assertEnumeratesCollection($collection, 0, $executor);
-    }
-
-    protected function assertEnumerates($count, callable $executor)
-    {
-        $this->assertEnumeratesCollection(
-            LazyCollection::times(100),
-            $count,
-            $executor
-        );
-    }
-
-    protected function assertEnumeratesCollection(
-        LazyCollection $collection,
-        $count,
-        callable $executor
-    ) {
-        $enumerated = 0;
-
-        $data = $this->countEnumerations($collection, $enumerated);
-
-        $executor($data);
-
-        $this->assertEnumerations($count, $enumerated);
-    }
-
-    protected function assertEnumeratesOnce(callable $executor)
-    {
-        $this->assertEnumeratesCollectionOnce(LazyCollection::times(10), $executor);
-    }
-
-    protected function assertEnumeratesCollectionOnce(
-        LazyCollection $collection,
-        callable $executor
-    ) {
-        $enumerated = 0;
-        $count = $collection->count();
-        $collection = $this->countEnumerations($collection, $enumerated);
-
-        $executor($collection);
-
-        $this->assertEquals(
-            $count,
-            $enumerated,
-            $count > $enumerated ? 'Failed to enumerate in full.' : 'Enumerated more than once.'
-        );
-    }
-
-    protected function assertEnumerations($expected, $actual)
-    {
-        $this->assertEquals(
-            $expected,
-            $actual,
-            "Failed asserting that {$actual} items that were enumerated matches expected {$expected}."
-        );
-    }
-
-    protected function countEnumerations(LazyCollection $collection, &$count)
-    {
-        return $collection->tapEach(function () use (&$count) {
-            $count++;
-        });
+        try {
+            $callback();
+        } catch (Exception $e) {
+            // Silence is golden
+        }
     }
 }
