@@ -7,7 +7,10 @@ use ErrorException;
 use Illuminate\Container\Container;
 use Illuminate\Contracts\Events\Dispatcher as DispatcherContract;
 use Illuminate\Contracts\View\Engine;
+use Illuminate\Contracts\View\View as ViewContract;
 use Illuminate\Events\Dispatcher;
+use Illuminate\Filesystem\Filesystem;
+use Illuminate\Support\HtmlString;
 use Illuminate\View\Compilers\CompilerInterface;
 use Illuminate\View\Engines\CompilerEngine;
 use Illuminate\View\Engines\EngineResolver;
@@ -59,6 +62,16 @@ class ViewFactoryTest extends TestCase
         $this->assertTrue($factory->exists('bar'));
     }
 
+    public function testRenderingOnceChecks()
+    {
+        $factory = $this->getFactory();
+        $this->assertFalse($factory->hasRenderedOnce('foo'));
+        $factory->markAsRenderedOnce('foo');
+        $this->assertTrue($factory->hasRenderedOnce('foo'));
+        $factory->flushState();
+        $this->assertFalse($factory->hasRenderedOnce('foo'));
+    }
+
     public function testFirstCreatesNewViewInstanceWithProperPath()
     {
         unset($_SERVER['__test.view']);
@@ -75,6 +88,7 @@ class ViewFactoryTest extends TestCase
         $factory->addExtension('php', 'php');
         $view = $factory->first(['bar', 'view'], ['foo' => 'bar'], ['baz' => 'boom']);
 
+        $this->assertInstanceOf(ViewContract::class, $view);
         $this->assertSame($engine, $view->getEngine());
         $this->assertSame($_SERVER['__test.view'], $view);
 
@@ -337,11 +351,11 @@ class ViewFactoryTest extends TestCase
     {
         $factory = $this->getFactory();
         $factory->getFinder()->shouldReceive('find')->andReturn(__DIR__.'/fixtures/component.php');
-        $factory->getEngineResolver()->shouldReceive('resolve')->andReturn(new PhpEngine);
+        $factory->getEngineResolver()->shouldReceive('resolve')->andReturn(new PhpEngine(new Filesystem));
         $factory->getDispatcher()->shouldReceive('dispatch');
         $factory->startComponent('component', ['name' => 'Taylor']);
         $factory->slot('title');
-        $factory->slot('website', 'laravel.com');
+        $factory->slot('website', 'laravel.com', []);
         echo 'title<hr>';
         $factory->endSlot();
         echo 'component';
@@ -349,6 +363,51 @@ class ViewFactoryTest extends TestCase
         $this->assertSame('title<hr> component Taylor laravel.com', $contents);
     }
 
+    public function testComponentHandlingUsingViewObject()
+    {
+        $factory = $this->getFactory();
+        $factory->getFinder()->shouldReceive('find')->andReturn(__DIR__.'/fixtures/component.php');
+        $factory->getEngineResolver()->shouldReceive('resolve')->andReturn(new PhpEngine(new Filesystem));
+        $factory->getDispatcher()->shouldReceive('dispatch');
+        $factory->startComponent($factory->make('component'), ['name' => 'Taylor']);
+        $factory->slot('title');
+        $factory->slot('website', 'laravel.com', []);
+        echo 'title<hr>';
+        $factory->endSlot();
+        echo 'component';
+        $contents = $factory->renderComponent();
+        $this->assertSame('title<hr> component Taylor laravel.com', $contents);
+    }
+
+    public function testComponentHandlingUsingClosure()
+    {
+        $factory = $this->getFactory();
+        $factory->getFinder()->shouldReceive('find')->andReturn(__DIR__.'/fixtures/component.php');
+        $factory->getEngineResolver()->shouldReceive('resolve')->andReturn(new PhpEngine(new Filesystem));
+        $factory->getDispatcher()->shouldReceive('dispatch');
+        $factory->startComponent(function ($data) use ($factory) {
+            $this->assertArrayHasKey('name', $data);
+            $this->assertSame($data['name'], 'Taylor');
+
+            return $factory->make('component');
+        }, ['name' => 'Taylor']);
+        $factory->slot('title');
+        $factory->slot('website', 'laravel.com', []);
+        echo 'title<hr>';
+        $factory->endSlot();
+        echo 'component';
+        $contents = $factory->renderComponent();
+        $this->assertSame('title<hr> component Taylor laravel.com', $contents);
+    }
+
+    public function testComponentHandlingUsingHtmlable()
+    {
+        $factory = $this->getFactory();
+        $factory->startComponent(new HtmlString('laravel.com'));
+        $contents = $factory->renderComponent();
+        $this->assertSame('laravel.com', $contents);
+    }
+
     public function testTranslation()
     {
         $container = new Container;
@@ -384,6 +443,27 @@ class ViewFactoryTest extends TestCase
         $this->assertSame('hi, Hello!', $factory->yieldPushContent('foo'));
     }
 
+    public function testSingleStackPrepend()
+    {
+        $factory = $this->getFactory();
+        $factory->startPrepend('foo');
+        echo 'hi';
+        $factory->stopPrepend();
+        $this->assertSame('hi', $factory->yieldPushContent('foo'));
+    }
+
+    public function testMultipleStackPrepend()
+    {
+        $factory = $this->getFactory();
+        $factory->startPrepend('foo');
+        echo ', Hello!';
+        $factory->stopPrepend();
+        $factory->startPrepend('foo');
+        echo 'hi';
+        $factory->stopPrepend();
+        $this->assertSame('hi, Hello!', $factory->yieldPushContent('foo'));
+    }
+
     public function testSessionAppending()
     {
         $factory = $this->getFactory();
@@ -442,6 +522,17 @@ class ViewFactoryTest extends TestCase
         $this->assertFalse($factory->hasSection('bar'));
     }
 
+    public function testSectionMissing()
+    {
+        $factory = $this->getFactory();
+        $factory->startSection('foo');
+        echo 'hello world';
+        $factory->stopSection();
+
+        $this->assertTrue($factory->sectionMissing('bar'));
+        $this->assertFalse($factory->sectionMissing('foo'));
+    }
+
     public function testGetSection()
     {
         $factory = $this->getFactory();
@@ -488,7 +579,7 @@ class ViewFactoryTest extends TestCase
         $this->expectException(ErrorException::class);
         $this->expectExceptionMessage('section exception message');
 
-        $engine = new CompilerEngine(m::mock(CompilerInterface::class));
+        $engine = new CompilerEngine(m::mock(CompilerInterface::class), new Filesystem);
         $engine->getCompiler()->shouldReceive('getCompiledPath')->andReturnUsing(function ($path) {
             return $path;
         });
@@ -572,7 +663,8 @@ class ViewFactoryTest extends TestCase
     {
         $factory = $this->getFactory();
 
-        $data = (new class {
+        $data = (new class
+        {
             public function generate()
             {
                 for ($count = 0; $count < 3; $count++) {
